// Verschiedene Funktionen fuer Astronomie-Programme
// Thorsten Lange, tl@thola.de

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <time.h>

#include "astrotools.h"

TTimeZone    JulDatZone = _MEZ;    				  /* gewhlte Zeitzone fr die JulDat-Funktionen */
TTimeDisplay JulDatShow = _TimeDisplay_Normal; /* gewhlte Anzeigeart des Datums */


const double JulDatZeitDifferenz[10] = { 0, 1.0/24, 2.0/24, 2.0/24, -5.0/24, -6.0/24, -7.0/24, -8.0/24 };
const char GetDatZoneStr[10][5] = {"UT","MEZ","MESZ","OEZ","EST","CST","MST","PST"};
const char SpektrumKlasse[8] = "OBAFGKM";


char *GetDateStr (char *dateStr)
{
	time_t     t;
	struct tm *ptm;
	int        year;
	time(&t);
	ptm  = localtime(&t);
	year = ptm->tm_year;
	year+= (year>69) ? 1900 : 2000;
	sprintf(dateStr,"%02i.%02i.%04i",ptm->tm_mday,ptm->tm_mon+1,year);
	return dateStr;
}

char *GetTimeStr (char *timeStr)
{
	time_t     t;
	struct tm *ptm;
	time(&t);
	ptm = localtime(&t);
	sprintf(timeStr,"%02i.%02i",ptm->tm_hour,ptm->tm_min);
	return timeStr;
}

char *GetYesterdayStr (char *dateStr)
{
	GetDateStr(dateStr);
	if (dateStr[1]!='0')
		dateStr[1]--;
	else {
		dateStr[0]--;
		dateStr[1] = '9';
	}   
	strcat(dateStr,",");
	if (dateStr[0]==' ') 
		dateStr[0] = '0';
	return dateStr;
}

double GetTodaysJulDat()
{
	char dateStr[32];
	return Zeit2JulDat( GetDateStr(dateStr) );
}


int atoHell (char *hell)
{
	if (hell[0]=='<') 
   	return 10000+(int)(atof(hell+1)*100+0.5);
  	else
   	return (int)(atof(hell)*100+0.5);
}

/* allgemeine astronomische Funktionen */

double Zeit2JulDat (char *datum)
/* datum='tt.mm.jjjj~hh.mm' oder 'tt.mm.jjjj,hh.mm' oder 'tt.mm.jjjj.uuuuu'
    oder 'jjjj/mm/tt.uuuuu' */
{
	double jd,b;
	int t,m,j=0;

	if (strlen(datum)<10) {
		return 0;
	}
	for (t=0; datum[t]!='\0'; t++) {
		if (datum[t]==' ') datum[t] = '0';
	}
	if (datum[4]=='/') {
	 	t = atoi(datum+8);
	 	m = atoi(datum+5);
	 	j = atoi(datum);
	} else {
	 	t = atoi(datum);
	 	m = atoi(datum+3);
		j = atoi(datum+6);
	}
	if (m<=2) { j--; m+=12; }
	if ((t<15) && (m<11) && (j<1583)) b=-2.0;
	else b = floor(j*0.0025)-floor(j*0.01);
	jd = floor(365.25*j)+floor(30.6001*(m+1))+b+1720996.5+t;
	/* Stunden oder Nachkommaanteil */
	if (((datum[10]==',') || (datum[10]=='~')) && (datum[11]!='\0')) {
		sscanf(datum+11,"%lf",&b);
		jd += (floor(b)+(b-floor(b))*10.0/6.0)/24.0;
	} else if (datum[10]=='.') {
		sscanf(datum+10,"%lf",&b);
		jd += b;
	}
	jd -= JulDatZeitDifferenz[JulDatZone];
	return jd;
}

char *JulDat2Zeit (double jd, char *sz)
{
  double a,b,c,d,e,f,t,m,j,h;

  jd += JulDatZeitDifferenz[JulDatZone];
  jd += 0.000006; /* 0.5 Sekunden addieren */
  a = floor(jd+0.5);
  if (a<2299161.0) c = a+1524.0;
  else {
    b = floor((a-1867216.25)/36524.25);
    c = a+b-floor(b*0.25)+1525.0;
  }
  d = floor((c-122.1)/365.25);
  e = floor(365.25*d);
  f = floor((c-e)/30.6001);
  t = c-e-floor(30.6001*f)+frac(jd+0.5);
  m = floor(f-1.0-12.0*floor(f/14.0));
  j = d-4715.0-floor((7.000001+m)*0.1);
  h = frac(t)*24+0.005;
  switch (JulDatShow) {
  case _TimeDisplay_Normal :
    sprintf(sz, "%02i.%02i.%04i, %2i.%02ih", trunc(t), trunc(m), trunc(j), trunc(h), trunc(frac(h)*60));
    break;
  case _TimeDisplay_Astro : 
    sprintf(sz, "%i/%02i/%02.3f", trunc(j), trunc(m), t);
    break;
  }
  return sz;
}

char *Rekt2Str (double r, int secStellen, char *s)
{
	char h[4],m[4];
	sprintf(h,"%2ih",trunc(r/15));
	sprintf(m,"%02im",trunc(frac(r/15)*60));
	if (secStellen>0) 
		sprintf(s,"%s%s%*.*fs",h,m,2+secStellen,secStellen,frac(frac(r/15)*60)*60);
	else if (secStellen==0) 
		sprintf(s,"%s%s%02is",h,m,trunc(frac(frac(r/15)*60)*60+0.5));
	else if (secStellen==-1)
		sprintf(s,"%s%02im",h,trunc(frac(r/15)*60+0.5));
	else 
		sprintf(s,"%s%2.*fm",h,-1-secStellen,frac(r/15)*60);
	return s;
}

char *Dekl2Str (double d, int secStellen, char *s)
{  
	char g[5],m[5],vz;
	if (d<0) {
		vz = '-';
		d = -d;
	} else {
		vz = '+';
	}
	sprintf(g,"%c%2i",vz,trunc(d));
	sprintf(m,"%02i'",trunc(frac(d)*60));
	if (secStellen>0)
		sprintf(s,"%s%s%0*.*f\"",g,m,2+secStellen,secStellen,frac(frac(d)*60)*60);
	else if (secStellen==0)
		sprintf(s,"%s%s%02i\"",g,m,trunc(frac(frac(d)*60)*60+0.5));
	else if (secStellen==-1)
		sprintf(s,"%s%02i'",g,trunc(frac(d)*60+0.5));
	else
		sprintf(s,"%s%0*.*f'",g,1-secStellen,-1-secStellen,frac(d)*60);
	return s;
}


const char* Hell2Str( int hell100Mag )
{
	static char str[10];
	if (hell100Mag < 10000)
		sprintf( str, " %4.1f", (double)hell100Mag * 0.01 );
	else
		sprintf( str, "<%4.1f", (double)(hell100Mag-10000) * 0.01 );
	return str;
}


int GetSpektrumNr (char *s)
{
	char *sk;
	sk = strchr(SpektrumKlasse,toupper(s[0]));
	if (sk!=NULL) return 10*(sk-SpektrumKlasse)+(atoi(s+1) % 10);
	else return 0;
}

char *Spektrum (int sknr, char *s)
{
	if (sknr==0) sprintf(s,"  ");
	else sprintf(s,"%c%i",SpektrumKlasse[sknr/10],sknr%10);
	return s;
}


char *Winkel2HMS (double w, char *s)
{
	sprintf(s,"%02fh%02fm%02fs", floor(w/15), floor((w-15*floor(w/15))*60),
                                (w*4-floor(w*4))*60);
	return s;
}

char *Winkel2Grad (double w, char *s)
{
	char wvz;
	if (w<0) wvz='-';
	else wvz='+';
	w = fabs(w);
	sprintf(s,"%c%02f%02f'%02f\"", wvz, floor(w), floor((w-floor(w))*60),
	                               (60*w-floor(60*w))*60);
	return s;
}


const char* Entfernung2Str (double distance)
{
	static char str[32];
	if (distance < 0.1) {
		distance *= 9.4605e12;
		if (distance > 1e6)
			sprintf( str, "%6.3g AE", distance / 149.6e6 );
		else
			sprintf( str, "%6.0f km", distance );
	} else {
		if (distance < 3261.6)
			sprintf( str, "%5.1g Lj", distance );
		else
			sprintf( str, "%6.1g kpc", distance / 3251.6 );
	}
	return str;
}


const char* ListOfConstellations[] = {
"And", "Andromedae",
"Ant", "Antilae",
"Apo", "Apodis",
"Aqr", "Aquarii",
"Aql", "Aquilae",
"Ara", "Arae",
"Ari", "Arietis",
"Aur", "Aurigae",
"Boo", "Bootis",
"Cae", "Caeli",
"Cam", "Camelopardalis",
"Cnc", "Cancri",
"CVn", "Canum Venaticorum",
"CMa", "Canis Majoris",
"CMi", "Canis Minoris", 
"Cap", "Capricorni",
"Car", "Carinae",
"Cas", "Cassiopeia",
"Cen", "Centauri",
"Cep", "Cephei",
"Cet", "Ceti",
"Cha", "Chamaeleonis",
"Cir", "Circini",
"Col", "Columbae",
"Com", "Comae Berenicis",
"CrA", "Coronae Australis",
"CrB", "Coronae Borealis",
"Crv", "Corvi",
"Crt", "Crateris",
"Cru", "Crucis",
"Cyg", "Cygni",
"Del", "Delphini",
"Dor", "Doradus",
"Dra", "Draconis",
"Equ", "Equulei",
"Eri", "Eridani",
"For", "Fornacis",
"Gem", "Geminorum",
"Gru", "Gruis",
"Her", "Herculis",
"Hor", "Horologii",
"Hya", "Hydrae",
"Hyi", "Hydri",
"Ind", "Indi",
"Lac", "Lacertae",
"Leo", "Leonis",
"LMi", "Leonis Minoris",
"Lep", "Leporis",
"Lib", "Librae",
"Lup", "Lupi",
"Lyn", "Lyncis",
"Lyr", "Lyrae",
"Men", "Mensae",
"Mic", "Microscopii",
"Mon", "Monocerotis",
"Mus", "Muscae",
"Nor", "Normae",
"Oct", "Octantis",
"Oph", "Ophiuchi",
"Ori", "Orionis",
"Pav", "Pavonis",
"Peg", "Pegasi",
"Per", "Persei",
"Pho", "Phoenicis",
"Pic", "Pictoris",
"Psc", "Piscium",
"PsA", "Piscis Austrini",
"Pup", "Puppis",
"Pyx", "Pyxidis",
"Ret", "Reticuli",
"Sge", "Sagittae",
"Sgr", "Sagittarii",
"Sco", "Scorpii",
"Scl", "Sculptoris",
"Sct", "Scuti",
"Ser", "Serpentis Caput",
"Sea", "Serpentis Cauda",
"Sex", "Sextantis",
"Tau", "Tauri",
"Tel", "Telescopii",
"Tri", "Trianguli",
"TrA", "Trianguli Australis",
"Tuc", "Tucanae",
"UMa", "Ursae Majoris",
"UMi", "Ursae Minoris",
"Vel", "Velorum",
"Vir", "Virginis",
"Vol", "Volantis",
"Vul", "Vulpeculae",
"NGC", "NGC",
"IC",  "IC",
"", "" };

int ConstellationMax = -1;

void InitConstellationList()
{
	if (ConstellationMax<0)
	{
		ConstellationMax = 0;
		while (ListOfConstellations[2*ConstellationMax][0]!=0)
			ConstellationMax++;
	}
}

const int GetConstellationMax()	
{	
	InitConstellationList();
	return ConstellationMax; 
}

const char* GetConstellationName( int theConstellationN )
{
	InitConstellationList();
	if (theConstellationN>=0 && theConstellationN<ConstellationMax)
		return ListOfConstellations[2*theConstellationN+1];
	return "";
}

const char* GetConstellationShort( int theConstellationN )
{
	InitConstellationList();
	if (theConstellationN>=0 && theConstellationN<ConstellationMax)
		return ListOfConstellations[2*theConstellationN];
	return "";
}


