
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "libtl_picture.h"

#define boolean char
#define TRUE  (1)
#define FALSE (0)

#define picture_get(pic,x,y)            pic->data[(x)+(y)*pic->width]

#define picture_set(pic,x,y,c)          pic->data[(x)+(y)*pic->width] = (c)

#define picture_set_test(pic,x,y,c)     if ((x>=0) && (x<pic->width) && \
                                            (y>=0) && (y<pic->height)) \
                                           picture_set(pic,x,y,c)


int            font_height;
unsigned char *font_data;


/******************************************************************************
							from here: wrbmp.c
******************************************************************************/

#define MEMZERO(d,s)	memset(d,0,s)

/*
 * Since BMP stores scanlines bottom-to-top, we have to invert the image
 * from JPEG's top-to-bottom order.  To do this, we save the outgoing data
 * in a virtual array during put_pixel_row calls, then actually emit the
 * BMP file during finish_output.  The virtual array contains one JSAMPLE per
 * pixel if the output is grayscale or colormapped, three if it is full color.
 */

/* Private version of data destination object */

typedef struct {
	FILE *output_file;
	long data_width;	/* JSAMPLEs per row */
	long row_width;		/* physical width of one row in the BMP file */
	int  pad_bytes;		/* number of padding bytes needed per row */
	long cur_output_row;	/* next row# to write to virtual array */
} dest_struct;

typedef dest_struct * dest_ptr;


/* Forward declarations */
void write_bmp_colormap (Picture *pic,dest_ptr dest);


void write_bmp_header (Picture *pic, dest_ptr dest)
/* Write a Windows-style BMP file header, including colormap if needed */
{
	char bmpfileheader[14];
	char bmpinfoheader[40];
#define PUT_2B(array,offset,value)  \
	(array[offset] = (char) ((value) & 0xFF), \
	 array[offset+1] = (char) (((value) >> 8) & 0xFF))
#define PUT_4B(array,offset,value)  \
	(array[offset] = (char) ((value) & 0xFF), \
	 array[offset+1] = (char) (((value) >> 8) & 0xFF), \
	 array[offset+2] = (char) (((value) >> 16) & 0xFF), \
	 array[offset+3] = (char) (((value) >> 24) & 0xFF))
	long headersize, bfSize;

	/* Compute colormap size and total file size */
	/* File size */
	headersize = 14 + 40 + 256 * 4; /* Header and colormap */
	bfSize = headersize + dest->row_width * pic->height;

	/* Set unused fields of header to 0 */
	MEMZERO(bmpfileheader, sizeof(bmpfileheader));
	MEMZERO(bmpinfoheader, sizeof(bmpinfoheader));

	/* Fill the file header */
	bmpfileheader[0] = 0x42;	/* first 2 bytes are ASCII 'B', 'M' */
	bmpfileheader[1] = 0x4D;
	PUT_4B(bmpfileheader, 2, bfSize); /* bfSize */
	/* we leave bfReserved1 & bfReserved2 = 0 */
	PUT_4B(bmpfileheader, 10, headersize); /* bfOffBits */

	/* Fill the info header (Microsoft calls this a BITMAPINFOHEADER) */
	PUT_2B(bmpinfoheader, 0, 40);	/* biSize */
	PUT_4B(bmpinfoheader, 4, pic->width); /* biWidth */
	PUT_4B(bmpinfoheader, 8, pic->height); /* biHeight */
	PUT_2B(bmpinfoheader, 12, 1);	/* biPlanes - must be 1 */
	PUT_2B(bmpinfoheader, 14, 8); /* biBitCount */
	/* we leave biCompression = 0, for none */
	/* we leave biSizeImage = 0; this is correct for uncompressed data */
	PUT_2B(bmpinfoheader, 32, 256); /* biClrUsed */
	/* we leave biClrImportant = 0 */

	fwrite(bmpfileheader, sizeof(bmpfileheader), 1, dest->output_file);
	fwrite(bmpinfoheader, sizeof(bmpinfoheader), 1, dest->output_file);

	write_bmp_colormap(pic,dest);
}


/*
 * Write the colormap.
 * Windows uses BGR0 map entries; OS/2 uses BGR entries.
 */

void write_bmp_colormap (Picture *pic, dest_ptr dest)
{
	FILE * outfile = dest->output_file;
	int i;

	/* Normal case with RGB colormap */
	for (i = 0; i < 256; i++) 
	{
		putc(pic->colormap[0][i], outfile);
		putc(pic->colormap[1][i], outfile);
		putc(pic->colormap[2][i], outfile);
		putc(0, outfile);
	}
}


void
finish_output_bmp (Picture *pic, dest_ptr dest)
{
	FILE * outfile = dest->output_file;
	long row;
	char *data_ptr;
	long col;

	/* Write the file body from our virtual array */
	for (row = pic->height; row > 0; row--) {
		data_ptr = (char*)&(pic->data[row * pic->width]);
		for (col = dest->row_width; col > 0; col--) {
			putc(*data_ptr, outfile);
			data_ptr++;
		}
	}

	/* Make sure we wrote the output file OK */
	fflush(outfile);
	if (ferror(outfile)) {}
}


/*
 * The module selection routine for BMP format output.
 */

dest_ptr init_write_bmp (Picture *pic)
{
	dest_ptr dest;
	long row_width;

	/* Create module interface object, fill in method pointers */
	dest = (dest_ptr) malloc(sizeof(dest_struct));

	/* Calculate output image dimensions so we can allocate space */

	/* Determine width of rows in the BMP file (padded to 4-byte boundary). */
	row_width = pic->width;
	dest->data_width = row_width;
	while ((row_width & 3) != 0) row_width++;
	dest->row_width = row_width;
	dest->pad_bytes = (int) (row_width - dest->data_width);

	/* Allocate space for inversion array, prepare for write pass */
	dest->cur_output_row = 0;

	/* Create decompressor output buffer. */

	return (dest_ptr) dest;
}

/******************************************************************************
							from here: wrgif.c
******************************************************************************/

#define	MAX_LZW_BITS	12	/* maximum LZW code size (4096 symbols) */

#define LZW_TABLE_SIZE	((int) 1 << MAX_LZW_BITS)

#define HSIZE		5003	/* hash table size for 80% occupancy */

#define MAXCODE(n_bits)	(((int) 1 << (n_bits)) - 1)


/*
 * The LZW hash table consists of two parallel arrays:
 *   hash_code[i]	code of symbol in slot i, or 0 if empty slot
 *   hash_value[i]	symbol's value; undefined if empty slot
 * where slot values (i) range from 0 to HSIZE-1.  The symbol value is
 * its prefix symbol's code concatenated with its suffix character.
 *
 * Algorithm:  use open addressing double hashing (no chaining) on the
 * prefix code / suffix character combination.  We do a variant of Knuth's
 * algorithm D (vol. 3, sec. 6.4) along with G. Knott's relatively-prime
 * secondary probe.
 *
 * The hash_value[] table is allocated from FAR heap space since it would
 * use up rather a lot of the near data space in a PC.
 */

typedef long hash_entry;	/* must hold (code_int<<8) | byte */

#define HASH_ENTRY(prefix,suffix)  ((((hash_entry) (prefix)) << 8) | (suffix))


/* Private version of data destination object */

typedef struct {
	dest_ptr pub;

	/* State for packing variable-width codes into a bitstream */
	int n_bits;			/* current number of bits/code */
	int maxcode;		/* maximum code, given n_bits */
	int init_bits;		/* initial n_bits ... restored after clear */
	long cur_accum;		/* holds bits not yet output */
	int cur_bits;			/* # of bits in cur_accum */

	/* LZW string construction */
	int waiting_code;	/* symbol not yet output; may be extendable */
	boolean first_byte;		/* if TRUE, waiting_code is not valid */

	/* State for LZW code assignment */
	int ClearCode;		/* clear code (doesn't change) */
	int EOFCode;		/* EOF code (ditto) */
	int free_code;		/* first not-yet-used symbol code */

	/* LZW hash table */
	int *hash_code;		/* => hash table of symbol codes */
	hash_entry *hash_value;	/* => hash table of symbol values */

	/* GIF data packet construction buffer */
	int bytesinpkt;		/* # of bytes in current packet */
	char packetbuf[256];		/* workspace for accumulating packet */

} gif_dest_struct;

typedef gif_dest_struct * gif_dest_ptr;


/*
 * Routines to package compressed data bytes into GIF data blocks.
 * A data block consists of a count byte (1..255) and that many data bytes.
 */

void flush_packet (gif_dest_ptr dinfo)
/* flush any accumulated data */
{
  int i;
  if (dinfo->bytesinpkt > 0) {	/* never write zero-length packet */
	dinfo->packetbuf[0] = (char) dinfo->bytesinpkt;
	for (i=0; i<=dinfo->bytesinpkt; i++)
	  putc(dinfo->packetbuf[i], dinfo->pub->output_file);
	dinfo->bytesinpkt = 0;
  }
}


/* Add a character to current packet; flush to disk if necessary */
#define CHAR_OUT(dinfo,c)  \
	{ (dinfo)->packetbuf[++(dinfo)->bytesinpkt] = (char) (c);  \
		 if ((dinfo)->bytesinpkt >= 255)  \
			flush_packet(dinfo);  \
	}


/* Routine to convert variable-width codes into a byte stream */

void output (gif_dest_ptr dinfo, int code)
/* Emit a code of n_bits bits */
/* Uses cur_accum and cur_bits to reblock into 8-bit bytes */
{
	dinfo->cur_accum |= ((long) code) << dinfo->cur_bits;
  dinfo->cur_bits += dinfo->n_bits;

	while (dinfo->cur_bits >= 8) {
	 CHAR_OUT(dinfo,dinfo->cur_accum & 0xFF);
	 dinfo->cur_accum >>= 8;
	 dinfo->cur_bits -= 8;
  }

  /*
	* If the next entry is going to be too big for the code size,
	* then increase it, if possible.  We do this here to ensure
	* that it's done in sync with the decoder's codesize increases.
	*/
	if (dinfo->free_code > dinfo->maxcode) {
	 dinfo->n_bits++;
	 if (dinfo->n_bits == MAX_LZW_BITS)
		dinfo->maxcode = LZW_TABLE_SIZE; /* free_code will never exceed this */
	 else
		dinfo->maxcode = MAXCODE(dinfo->n_bits);
  }
}


/* The LZW algorithm proper */


void clear_hash (gif_dest_ptr dinfo)
/* Fill the hash table with empty entries */
{
  /* It's sufficient to zero hash_code[] */
  MEMZERO(dinfo->hash_code, HSIZE * sizeof(int));
}


void clear_block (gif_dest_ptr dinfo)
/* Reset compressor and issue a Clear code */
{
  clear_hash(dinfo);			/* delete all the symbols */
  dinfo->free_code = dinfo->ClearCode + 2;
  output(dinfo, dinfo->ClearCode);	/* inform decoder */
	dinfo->n_bits = dinfo->init_bits;	/* reset code size */
  dinfo->maxcode = MAXCODE(dinfo->n_bits);
}


void compress_init (gif_dest_ptr dinfo, int i_bits)
/* Initialize LZW compressor */
{
  /* init all the state variables */
	dinfo->n_bits = dinfo->init_bits = i_bits;
  dinfo->maxcode = MAXCODE(dinfo->n_bits);
  dinfo->ClearCode = ((int) 1 << (i_bits - 1));
  dinfo->EOFCode = dinfo->ClearCode + 1;
  dinfo->free_code = dinfo->ClearCode + 2;
  dinfo->first_byte = TRUE;	/* no waiting symbol yet */
  /* init output buffering vars */
  dinfo->bytesinpkt = 0;
  dinfo->cur_accum = 0;
  dinfo->cur_bits = 0;
  /* clear hash table */
  clear_hash(dinfo);
  /* GIF specifies an initial Clear code */
  output(dinfo, dinfo->ClearCode);
}


void compress_byte (gif_dest_ptr dinfo, int c)
/* Accept and compress one 8-bit byte */
{
  int i;
  int disp;
  hash_entry probe_value;

  if (dinfo->first_byte) {	/* need to initialize waiting_code */
	 dinfo->waiting_code = c;
	 dinfo->first_byte = FALSE;
	 return;
  }

	/* Probe hash table to see if a symbol exists for
	* waiting_code followed by c.
	* If so, replace waiting_code by that symbol and return.
	*/
  i = (c << (MAX_LZW_BITS-8)) + dinfo->waiting_code;
  /* i is less than twice 2**MAX_LZW_BITS, therefore less than twice HSIZE */
	if (i >= HSIZE)
	 i -= HSIZE;

  probe_value = HASH_ENTRY(dinfo->waiting_code, c);

  if (dinfo->hash_code[i] != 0) { /* is first probed slot empty? */
	 if (dinfo->hash_value[i] == probe_value) {
		dinfo->waiting_code = dinfo->hash_code[i];
		return;
	 }
	 if (i == 0)			/* secondary hash (after G. Knott) */
		disp = 1;
	 else
		disp = HSIZE - i;
	 for (;;) {
		i -= disp;
		if (i < 0)
			i += HSIZE;
		if (dinfo->hash_code[i] == 0)
			break;			/* hit empty slot */
		if (dinfo->hash_value[i] == probe_value) {
			dinfo->waiting_code = dinfo->hash_code[i];
			return;
		}
	 }
  }

	/* here when hashtable[i] is an empty slot; desired symbol not in table */
  output(dinfo, dinfo->waiting_code);
  if (dinfo->free_code < LZW_TABLE_SIZE) {
	 dinfo->hash_code[i] = dinfo->free_code++; /* add symbol to hashtable */
	 dinfo->hash_value[i] = probe_value;
  } else
	 clear_block(dinfo);
  dinfo->waiting_code = c;
}


void compress_term (gif_dest_ptr dinfo)
/* Clean up at end */
{
  /* Flush out the buffered code */
  if (! dinfo->first_byte)
	 output(dinfo, dinfo->waiting_code);
  /* Send an EOF code */
  output(dinfo, dinfo->EOFCode);
  /* Flush the bit-packing buffer */
  if (dinfo->cur_bits > 0) {
	 CHAR_OUT(dinfo, dinfo->cur_accum & 0xFF);
  }
  /* Flush the packet buffer */
  flush_packet(dinfo);
}


/* GIF header construction */


void put_word (gif_dest_ptr dinfo, unsigned int w)
/* Emit a 16-bit word, LSB first */
{
  putc((char)(w & 0xFF), dinfo->pub->output_file);
  putc((char)((w >> 8) & 0xFF), dinfo->pub->output_file);
}


void put_3bytes (gif_dest_ptr dinfo, int val)
/* Emit 3 copies of same byte value --- handy subr for colormap construction */
{
	putc((char)val, dinfo->pub->output_file);
	putc((char)val, dinfo->pub->output_file);
	putc((char)val, dinfo->pub->output_file);
}


void emit_header (Picture *pic, gif_dest_ptr dinfo)
/* Output the GIF file header, including color map */
{
	int i, FlagByte;

	/*
	* Write the GIF header.
	* Note that we generate a plain GIF87 header for maximum compatibility.
	*/
	putc('G', dinfo->pub->output_file);
	putc('I', dinfo->pub->output_file);
	putc('F', dinfo->pub->output_file);
	putc('8', dinfo->pub->output_file);
	putc('7', dinfo->pub->output_file);
	putc('a', dinfo->pub->output_file);
	/* Write the Logical Screen Descriptor */
	put_word(dinfo, (unsigned int) pic->width);
	put_word(dinfo, (unsigned int) pic->height);
	FlagByte = 0x80;		/* Yes, there is a global color table */
	FlagByte |= (8-1) << 4; /* color resolution */
	FlagByte |= (8-1);	/* size of global color table */
	putc((char)FlagByte, dinfo->pub->output_file);
	putc(0, dinfo->pub->output_file); /* Background color index */
	putc(0, dinfo->pub->output_file); /* Reserved (aspect ratio in GIF89) */
	/* Write the Global Color Map */
	/* If the color map is more than 8 bits precision, */
	/* we reduce it to 8 bits by shifting */
	for (i=0; i < 256; i++) 
	{
		/* Normal case: RGB color map */
		putc(pic->colormap[2][i], dinfo->pub->output_file);
		putc(pic->colormap[1][i], dinfo->pub->output_file);
		putc(pic->colormap[0][i], dinfo->pub->output_file);
	}
	/* Write image separator and Image Descriptor */
	putc(',', dinfo->pub->output_file); /* separator */
	put_word(dinfo, 0);		/* left/top offset */
	put_word(dinfo, 0);
	put_word(dinfo, (unsigned int) pic->width); /* image size */
	put_word(dinfo, (unsigned int) pic->height);
	/* flag byte: not interlaced, no local color map */
	putc(0x00, dinfo->pub->output_file);
	/* Write Initial Code Size byte */
	putc((char)8, dinfo->pub->output_file);

	/* Initialize for LZW compression of image data */
	compress_init(dinfo, 8+1);
}


/*
 * Write some pixel data.
 * In this module rows_supplied will always be 1.
 */

void put_pixel_rows (Picture *pic, gif_dest_ptr dinfo, int row)
{
	Pixel *ptr;
	int col;

	ptr = &pic->data[(long)row * pic->width];
	for (col = pic->width; col > 0; col--) {
		compress_byte(dinfo, (int)*ptr);
		ptr++;
	}
}


/*
 * Finish up at the end of the file.
 */

void finish_output_gif (gif_dest_ptr dinfo)
{
	/* Flush LZW mechanism */
	compress_term(dinfo);
	/* Write a zero-length data block to end the series */
	putc(0, dinfo->pub->output_file);
	/* Write the GIF terminator mark */
	putc(';', dinfo->pub->output_file);
	/* Make sure we wrote the output file OK */
	fflush(dinfo->pub->output_file);
	/*
	if (ferror(dinfo->pub->output_file))
	 ERREXIT(cinfo, JERR_FILE_WRITE);
	*/
}


/*
 * The module selection routine for GIF format output.
 */

gif_dest_ptr init_write_gif (void)
{
	gif_dest_ptr dest;

	/* Create module interface object, fill in method pointers */
	dest = (gif_dest_ptr) malloc(sizeof(gif_dest_struct));
	dest->pub = (dest_ptr) malloc(sizeof(dest_struct));

	/* Allocate space for hash table */
	dest->hash_code = (int *) malloc(HSIZE * sizeof(int));
	dest->hash_value = (hash_entry *) malloc(HSIZE * sizeof(hash_entry));

	return (gif_dest_ptr) dest;
}

/*****************************************************************************
								Picture-Functions
*****************************************************************************/

short picture_font_load (const char *name)
{
	FILE *fFont;
	short id;
	
	fFont = fopen(name,"rb");
	if (fFont==NULL) {
		return FALSE;
	}
	fread(&id,sizeof(short),1,fFont);
	if (id==0x0436) {
  		/* mode = */ fgetc(fFont);
  		font_height = (int)fgetc(fFont);
	} else {
		fprintf(stderr,"Warnung: %s ist kein .PSF-File\n",name); 
		fseek(fFont,0,SEEK_SET);
		font_height = atoi(name+strlen(name)-2);
		if (font_height==0)
			font_height = atoi(name+strlen(name)-1);
		if (font_height==0) {
			fprintf(stderr,"Fehler: Kann Zeichengre nicht bestimmen!\n");
			return FALSE;
		}	
	}	
	id = 256*font_height*sizeof(char);
	font_data = (char*) malloc(id);
	fread(font_data,sizeof(char),id,fFont);
	fclose(fFont);
	return TRUE;
}

void picture_font_free (void)
{
	if (font_data!=NULL)
		free(font_data);
	font_data = NULL;	
}

Picture *picture_init (long width, long height, const char *name)
{
	Picture *pic;
	pic = (Picture*) malloc(sizeof(Picture));
	pic->data   = malloc(sizeof(Pixel)*width*height);
	pic->width  = width;
	pic->height = height;
	pic->x      = 0;
	pic->y      = 0;
	strcpy(pic->name,name);
	return pic;
}

void picture_free (Picture *pic)
{
	if (pic!=NULL)
		free(pic->data);
}

void picture_pixel (Picture *pic, long x, long y, Pixel color)
{
	picture_set_test(pic,x,y,color);
}

void picture_horline (Picture *pic, long y, Pixel color)
{
	long x;
	if ((y<0) || (y>=pic->height)) return;
	for (x=0; x<pic->width; x++)
		picture_set(pic,x,y,color);
}

void picture_vertline (Picture *pic, long x, Pixel color)
{
	long y;
	if ((x<0) || (x>=pic->width)) return;
	for (y=0; y<pic->height; y++)
		picture_set(pic,x,y,color);
}

void picture_line (Picture *pic, long x0, long y0, long x1, long y1, Pixel color)
{
	long xd,yd, xs,ys, xn,yn, i;
	double q;
	xs = (x0<x1) ? 1 : -1;
	ys = (y0<y1) ? 1 : -1;
	picture_set_test(pic,x0,y0,color);
	if (x0==x1) {
	 while (y0!=y1) {
		y0 += ys;
		picture_set_test(pic,x0,y0,color);
	 }
	} else if (y0==y1) {
	 while (x0!=x1) {
		x0 += xs;
		picture_set_test(pic,x0,y0,color);
	 }
	} else {
		q = (double)(abs(x1-x0)+1)/(double)(abs(y1-y0)+1);
		xn = (int)q;
		if (xn==0) xn=1;
		yn = (int)(1/q);
		if (yn==0) yn=1;
		while ((x0!=x1) && (y0!=y1)) {
			xd = abs(x1-x0);
			yd = abs(y1-y0);
			if (xd>yd) {
				for (i=0; (i<xn) && (x0!=x1); i++) {
					x0 += xs;
					picture_set_test(pic,x0,y0,color);
				}
				for (i=0; (i<yn) && (y0!=y1); i++) {
					y0 += ys;
					picture_set_test(pic,x0,y0,color);
				}
			} else {
				for (i=0; (i<yn) && (y0!=y1); i++) {
					y0 += ys;
					picture_set_test(pic,x0,y0,color);
				}
				for (i=0; (i<xn) && (x0!=x1); i++) {
					x0 += xs;
					picture_set_test(pic,x0,y0,color);
				}
			}
		}
	}
}

void picture_rect (Picture *pic, long x0, long y0, long x1, long y1, Pixel color)
{
	long x,y;
	if (x0>x1) { x=x0; x0=x1; x1=x; }
	if (y0>y1) { y=y0; y0=y1; y1=y; }
	for (x=x0; x<=x1; x++) 
	{
		picture_set_test(pic,x,y0,color);
		picture_set_test(pic,x,y1,color);
	}
	for (y=y0; y<=y1; y++) 
	{
		picture_set_test(pic,x0,y,color);
		picture_set_test(pic,x1,y,color);
	}
}

void picture_rect_fill (Picture *pic, long x0, long y0, long x1, long y1, Pixel color)
{
	long ix, iy;
	if (x0>x1) { ix=x0; x0=x1; x1=ix; }
	if (x0<0) x0=0;
	if (x1<0) x1=0;
	if (x0>=pic->width) x0=pic->width-1;
	if (x1>=pic->width) x1=pic->width-1;
	if (y0>y1) { iy=y0; y0=y1; y1=iy; }
	if (y0<0) y0=0;
	if (y1<0) y1=0;
	if (y0>pic->height) y0=pic->height;
	if (y1>pic->height) y1=pic->height;
	while (x0<x1) 
	{
		for (iy=y0; iy<y1; iy++)
			picture_set(pic,x0,iy,color);
		x0++;
	}
}

void picture_circle (Picture *pic, long x, long y, long radius, Pixel color)
{
	long ix,iy,my,w,v,r2;
	radius--;
	r2 = radius*radius;
	for (ix=radius; ix>=0; ix--) 
	{
		w = r2;
		my = 0;
		for (iy=radius; iy>=0; iy--) 
		{
			v = abs(ix*ix+iy*iy-r2);
			if (v<w) 
			{
				w = v;
				my = iy;
			}
		}
		picture_set_test(pic,x-ix,y-my,color);		
		picture_set_test(pic,x-ix,y+my,color);		
		picture_set_test(pic,x+ix,y+my,color);		
		picture_set_test(pic,x+ix,y-my,color);		
	}		
}


void picture_circle_fill (Picture *pic, long x, long y, long radius, Pixel color)
{
	long ix,iy,r2,rt;
	radius--;
	r2 = radius*radius;
	for (ix=-radius; ix<=radius; ix++) 
	{
		for (iy=-radius; iy<=radius; iy++) 
		{
			rt = abs(ix*ix+iy*iy);
			if (rt<=r2) 
			{
				picture_set_test(pic,x+ix,y+iy,color);		
			}
		}
	}		
}

void picture_vector (Picture *pic, long x0, long y0, long x1, long y1, Pixel color)
{
	long xv,yv;
	picture_line(pic,x0,y0,x1,y1,color);
	xv = (x1-x0)/2;
	yv = (y1-y0)/2;
	while ((xv>4) || (yv>4)) 
	{
		xv = (3*xv)/4;
		yv = (3*yv)/4;
	}
	picture_line(pic,x1-xv-yv,y1-yv+xv,x1,y1,color);
	picture_line(pic,x1-xv+yv,y1-yv-xv,x1,y1,color);
}

void picture_lineto (Picture *pic, long x1, long y1, Pixel color)
{
	picture_line(pic,pic->x,pic->y,x1,y1,color);
	pic->x = x1;
	pic->y = y1;
}

void picture_text_char (Picture *pic, long x, long y, Pixel color, boolean transparent, int size, int number)
{
	int x1,y1,sx,sy;
	unsigned char byte,bit;
	for (y1=0; y1<font_height; y1++) 
	{
		byte = font_data[number*font_height+y1]; 
		bit  = 128;
		for (x1=0; x1<8; x1++) 
		{
			if ((byte & bit)>0) 
			{
				for (sx=0; sx<size; sx++)
					for (sy=0; sy<size; sy++)
						picture_set_test(pic,x+x1*size+sx,y+y1*size+sy,color);
			} 
			else 
			{
				if (!transparent)
					for (sx=0; sx<size; sx++)
						for (sy=0; sy<size; sy++)
							picture_set_test(pic,x+x1*size+sx,y+y1*size+sy,0);
			}
			bit = bit >> 1;
		}		
	}
}

void picture_text (Picture *pic, long x, long y, Pixel color, int size, const char *text)
{
	int i = 0;
	while (text[i]!='\0') 
	{
		picture_text_char(pic,x,y,color,FALSE,size,(int)text[i]);
		i++;
		x += 8*size;
	}	
}

void picture_shift_hor (Picture *pic, long shift, Pixel color)
{
	long xt,xf,y;
	Pixel copy;

	if (shift<0) 
	{
		for (y=0; y<pic->height; y++) 
		{
			for (xt=pic->width+shift-1, xf=pic->width-1; xt>=0; xt--) 
			{
				copy = picture_get(pic,xf,y);
				picture_set(pic,xt,y,copy);
			}
			for (xt=pic->width+shift; xt<pic->width; xt++)
				picture_set(pic,xt,y,color);
		}
	} 
	else 
	{
		for (y=0; y<pic->height; y++) 
		{
			for (xt=shift, xf=0; xt<pic->width; xt++) 
			{
				copy = picture_get(pic,xf,y);
				picture_set(pic,xt,y,copy);
			}
			for (xt=0; xt<shift; xt++)
				picture_set(pic,xt,y,color);
		}
	}
}

void picture_colormap_entry (Picture *pic, int nr, unsigned long color)
{
	if ((nr<0) || (nr>255)) return;
	pic->colormap[0][nr] = (char)(color & 0xff);
	pic->colormap[1][nr] = (char)((color >> 8) & 0xff);
	pic->colormap[2][nr] = (char)((color >> 16) & 0xff);
}

void picture_colormap (Picture *pic, unsigned long *color)
{
	int nr;
	for (nr=255; nr>=0; nr--) 
	{
		pic->colormap[0][nr] = (char)( color[nr]        & 0xff);
		pic->colormap[1][nr] = (char)((color[nr] >>  8) & 0xff);
		pic->colormap[2][nr] = (char)((color[nr] >> 16) & 0xff);
	}
}

void write_picture_to_bmp (Picture *pic, char *name)
{
	dest_ptr dest;

	dest = init_write_bmp(pic);
	dest->output_file = fopen( (name!=NULL) ? name : pic->name, "wb");
	write_bmp_header(pic,dest);
	finish_output_bmp(pic,dest);
	fclose(dest->output_file);
	free(dest);
}

void write_picture_to_gif (Picture *pic, char *name)
{
	gif_dest_ptr dest;
	int i;

	dest = init_write_gif();
	if (dest==NULL) return;

	dest->pub->output_file = fopen( (name!=NULL) ? name : pic->name, "wb");
	emit_header(pic,dest);
	for (i=0; i<pic->height; i++) 
		put_pixel_rows(pic,dest,i);
	finish_output_gif(dest);
	fclose(dest->pub->output_file);
	free(dest->pub);
	free(dest->hash_code);
	free(dest->hash_value);
	free(dest);
}

void write_picture_to_ppm (Picture *pic, char *name)
{
	long x,y;
	FILE *ppm;

	ppm = fopen( (name!=NULL) ? name : pic->name, "w");
	fprintf(ppm,"P2\n");
	fprintf(ppm,"%ld %ld\n",pic->width,pic->height);
	fprintf(ppm,"255\n");
	for (y=0; y<pic->height; y++) 
	{
		for (x=0; x<pic->width; x++)
			fprintf(ppm,"%.4d ",pic->data[y*pic->width+x]);
		fprintf(ppm,"\n");
	}
	fclose(ppm);
}


