
#include <stdlib.h>
#include <forms.h>
#include "libtl/libtl_string.h"
#include "libtl/libtl_picture.h"
#include "libtl/libtl_draw.h"
#include "libtl/libtl_printer.h"

#include "vs_fd.h"
#include "vs_sub.h"
#include "vs_language.h"

#include "bitmaps/zoomin.h"
#include "bitmaps/zoomout.h"

FD_chart *fd_chart;

int     view_mag;
double  view_zoom=1.0, view_zoomX=1.0, view_zoomY=1.0;
bool    exit_when_close = FALSE;

#define MAXSTARS 1000

typedef struct {
	char  name[32];
	char  band;
	float scale;
	char  date[16];
	int   stars;
	struct {
		int x,y,mag,sat;
		char *name;
	} star[MAXSTARS];
	int xmin,xmax,ymin,ymax,magmin,magmax;
} TChart,*PChart;

PChart useChart = NULL;
 
PChart chart_load (char *filename)
{
	FILE    *f;
	char     str[80],*c;
	bool     weiter = TRUE;
	int      n;
	PChart   newchart;
	
	f = fopen(filename,"r");
	if (f==NULL)
		return NULL;

	while (weiter && fgetline(str,80,f)!=NULL) {
		if (strncmp(str,"frame ",6)==0)
			weiter = FALSE;
	}	
	if (weiter) return NULL;
	
	newchart = (PChart)malloc(sizeof(TChart));
	strn0cpy(newchart->name,str+6,31);
	newchart->band = (strlen(str)>41) ? str[41] : 0;

	if (fgetline(str,80,f)==NULL) {
		free(newchart);
		return NULL;
	}
	c = strstr(str,"SCALE");
	newchart->scale = (c==NULL) ? 0.02 : atof(c+6);
	strn0cpy(newchart->date,str+6,15);
	newchart->stars = 0;
	newchart->xmin = +999999;
	newchart->xmax = -999999;	
	newchart->ymin = +999999;
	newchart->ymax = -999999;	
	newchart->magmin = 0;
	newchart->magmax = 9999;	

	weiter = TRUE;
	do {
		if ((fgetline(str,80,f)==NULL) || (strncmp(str,"---",3)==0))
			weiter = FALSE;
		else {
			sscanf(str,"%i %i %i %i %n",
			           &newchart->star[newchart->stars].x,
			           &newchart->star[newchart->stars].y,
			           &newchart->star[newchart->stars].mag,
			           &newchart->star[newchart->stars].sat,
			           &n);
			c = str+n;
			n = strlen(str)-n;
			if (n>0) {
				newchart->star[newchart->stars].name = (char*)malloc(n+1);
				strn0cpy(newchart->star[newchart->stars].name,c,n);
			}
			testmax( newchart->star[newchart->stars].x, newchart->xmax);
			testmin( newchart->star[newchart->stars].x, newchart->xmin);
			testmax( newchart->star[newchart->stars].y, newchart->ymax);
			testmin( newchart->star[newchart->stars].y, newchart->ymin);
			testmax( newchart->star[newchart->stars].mag, newchart->magmin);
			testmin( newchart->star[newchart->stars].mag, newchart->magmax);
			newchart->stars++;
			weiter = (newchart->stars<MAXSTARS);
		}
	} while (weiter);

	return newchart;
}

void chart_delete (PChart chart)
{
	if (chart!=NULL) {
		while (chart->stars>0) {
			chart->stars--;
			if (chart->star[chart->stars].name != NULL)
				free(chart->star[chart->stars].name);
		}
		free(chart);
		chart = NULL;
	}	
}

void Chart_Paint (PChart chart, bool printChart)
{
	PDrawBitmap   db;
	int           wb,wh, x,y,r, i;
	float         fakX,fakY,zoomX,zoomY;
		
	if (chart==NULL) return;
	
	db = draw_init( printChart ? 0 : fd_chart->graph, &wb,&wh, "","");
	if (db==NULL) return;
  
  fakX = (float)(wb-10)/view_zoom;
  fakY = (float)(wh-10)/view_zoom;
  zoomX = fakX*view_zoomX*(1.0-view_zoom);
  zoomY = fakY*view_zoomY*(1.0-view_zoom);
  fakX /= (float)(chart->xmax-chart->xmin);
  fakY /= (float)(chart->ymax-chart->ymin);
  
  for (i=chart->stars-1; i>=0; i--) 
  	if (chart->star[i].mag <= view_mag) {
  		x = 5+(int)((float)(chart->star[i].x-chart->xmin)*fakX-zoomX);
  		y = 5+(int)((float)(chart->star[i].y-chart->ymin)*fakY-zoomY);
  		r = (view_mag-chart->star[i].mag)/100+1;
  		draw_circle(db,x,y,r,FL_WHITE);
  		if (chart->star[i].name!=NULL)
  			draw_text(db,x,y-r,FL_RED,chart->star[i].name);
  	}

	if (printChart) {
		char  str[128];
		FILE *fLst;
		fLst = OpenPrinter();
		sprintf(str,"Chart: %s",chart->name);
		x = 41-strlen(str)/2;
		while (x>0) {
			fprintf(fLst," ");
			x--;
		}
		fprintf(fLst,"%s\n",str);
		draw_rect(db,0,0,wb-1,wh-1,FL_WHITE);
		PrintScreen(fLst,db->pic->data,grafikDruckX,grafikDruckY,grafikDruckEinschubX,grafikDruckEinschubY);
		ClosePrinter(fLst);
	}
	
  draw_free(db);
}

void Chart_Close ()
{
  fd_chart_wp.x = fd_chart->chart->x;
  fd_chart_wp.y = fd_chart->chart->y;
  fd_chart_wp.w = fd_chart->chart->w;
  fd_chart_wp.h = fd_chart->chart->h;
  fl_hide_form(fd_chart->chart);
 	chart_delete(useChart);
 	useChart = NULL;
 	if (exit_when_close)
 		exit(1);
}

void chart_slider ()
{
	fl_set_slider_size (fd_chart->HorPos,view_zoom);
	fl_set_slider_value(fd_chart->HorPos,view_zoomX);
	fl_set_slider_size (fd_chart->VertPos,view_zoom);
	fl_set_slider_value(fd_chart->VertPos,view_zoomY);
}

void chart_init (char *filename)
{
	char str[64];
	double min;
	useChart = chart_load(filename);
	if (useChart!=NULL) {
		view_mag = useChart->magmin;
		min = (double)(view_mag/100+1);
		if (min<10.0) min=10.0;
		fl_set_slider_bounds(fd_chart->mag, 10.0,min);
		fl_set_slider_value(fd_chart->mag, min);
		view_zoom  = 1.0;
		view_zoomX = 1.0;
		view_zoomY = 1.0;
		chart_slider();
		sprintf(str,"Chart: %s",useChart->name);
		fl_set_form_title(fd_chart->chart,str);
	}
}

void cb_Chart (FL_OBJECT *ob, long mode)
{
	switch (mode) {
	case 1: /* Close */
		Chart_Close();
	  return;
	case 2: /* Load */
	 {char *ptr;
    ptr = (char*)fl_show_fselector("Load a chart-file",pfadChart,"*.cht","")		;
		if (ptr!=NULL) 
			chart_init(ptr);
		break;
	 }
	case 3: /* Print */
		Chart_Paint(useChart,TRUE);
		break;
	case 10: /* mag */
    view_mag = (int)(100*fl_get_slider_value(fd_chart->mag));
		break;
	case 11: /* zoom in */
	 {double f = 1.0-view_zoom;
		view_zoom *= 0.5;
		view_zoomX = (view_zoomX*f+view_zoom*0.5)/(1.0-view_zoom);
		view_zoomY = (view_zoomY*f+view_zoom*0.5)/(1.0-view_zoom);
		chart_slider();
		break;
	 }
	case 12: /* zoom out */
	 {double f = 1.0-view_zoom;
		if (view_zoom<0.5) {
			view_zoomX = (view_zoomX*f-view_zoom*0.5)/(1-2*view_zoom);
			view_zoomY = (view_zoomY*f-view_zoom*0.5)/(1-2*view_zoom);
			view_zoom *= 2.0;
		} else {
			view_zoom  = 1.0;
			view_zoomX = 1.0;
			view_zoomY = 1.0;
		}
		chart_slider();
		break;
	 }
	case 13: /* zoom vertical */
		view_zoomY = fl_get_slider_value(fd_chart->VertPos);
		break;
	case 14: /* zoom horizontal */
		view_zoomX = fl_get_slider_value(fd_chart->HorPos);
		break;
	}
 	fl_redraw_form(fd_chart->chart);
	Chart_Paint(useChart,FALSE);
}

int posthandler_Chart (FL_OBJECT *ob, int event, FL_Coord mx, FL_Coord my, int key, void *xev)
{
	switch (event) {
	case FL_DRAW:
		Chart_Paint(useChart,FALSE);
		break;
	}
	return 0;	
}

void Chart_Graph ()
{
	char str[80];
	
	if (fd_chart!=NULL && fl_form_is_visible(fd_chart->chart)) {
		fl_hide_form(fd_chart->chart);
	} else {
		fd_chart = create_form_chart();	
		fl_set_object_label( fd_chart->Load, LS_Load );
		fl_set_object_label( fd_chart->Print, LS_Print );
		fl_set_object_label( fd_chart->Close, LS_Close );
		fl_set_form_minsize(fd_chart->chart,450,300);
		fl_set_object_posthandler(fd_chart->graph, posthandler_Chart);
		fl_set_slider_precision(fd_chart->mag, 1);
		fl_set_slider_bounds(fd_chart->mag, 10.0,20.0);
		fl_set_slider_return(fd_chart->mag, FL_RETURN_CHANGED);
		fl_set_bitmapbutton_data(fd_chart->ZoomIn, zoomin_width,zoomin_height,zoomin_bits);
		fl_set_bitmapbutton_data(fd_chart->ZoomOut, zoomout_width,zoomout_height,zoomout_bits);
	}
	sprintf(str,"Chart: %s", (useChart!=NULL) ? useChart->name : "()");
	if (fd_chart_wp.x>0) {
		fl_set_form_position(fd_chart->chart, fd_chart_wp.x,fd_chart_wp.y);
		fl_set_form_size    (fd_chart->chart, fd_chart_wp.w,fd_chart_wp.h);
	}
	fl_show_form( fd_chart->chart,
                 ((fd_chart_wp.x>0) ? FL_PLACE_POSITION : FL_PLACE_MOUSE | FL_FREE_SIZE),
                 FL_FULLBORDER,str);
	fl_raise_form(fd_chart->chart);
}

void exit_chart()
{
	chart_delete(useChart);
	useChart = NULL;
}

void Script_Chart (int argc, char *argv[])
{
	exit_when_close = TRUE;
	atexit(exit_chart);

	fl_initialize(&argc, argv, 0, 0, 0);
	Chart_Graph();
	
	if (scriptArg[0]!='\0') 
	{
		char filename[128];
		sprintf(filename,"%s/%s.cht",pfadChart,scriptArg);
		chart_init(filename);
	}
		
	fl_do_forms();
}


void Simbad2Chart (const char* filename)
{	// TL 05.08.2001, create *.cht from Simbad object file
	bool weiter = TRUE;
	char str[256], chartname[256];
	int countN = 0;
	FILE *fOut;
	FILE *fIn = fopen(filename,"r");
	if (!fIn) {
		fprintf(stderr,"File %s not found\n",filename);
		return;
	}
	
	sprintf(chartname,"%s/%s.cht",pfadChart,filename);
	fOut = fopen(chartname,"w");
	if (!fOut){
		fprintf(stderr,"Can't create output file %s\n",str);
		fclose(fIn);
		return;
	}
	
	while (weiter && fgetline(str,120,fIn)!=NULL) {
		if (strncmp(str,"identifier ",11)==0)
			weiter = FALSE;
	}	
	if (weiter) {
		fclose(fIn);
		fclose(fOut);
		return;
	}
	
	fprintf(fOut,"frame %s                                              \n",filename);
	fprintf(fOut,"data (unknown)      \n");
	
	fgetline(str,120,fIn);
	weiter = TRUE;
	while (weiter && fgetline(str,120,fIn)!=NULL) {
		if (strlen(str)>70) {
			float x,y, mag;
			x = atof(str+33)*3600 + atof(str+36)*60 + atof(str+39);
			x = 24*3600-x;
			y = atof(str+46)*3600 + atof(str+50)*60;
			if (str[52]==' ')
				y += atof(str+53);
			mag = atof(str+(str[63]==' '?64:63));
			if (mag==0)
				mag = atof(str+(str[74]==' '?75:74));
			if (mag!=0) {
				fprintf(fOut," %i %i %i 0 ", (int)(x*10), (int)(y*10), (int)(mag*100+0.5) );
				if (str[0]=='V') {
					str[26] = 0;
					fprintf(fOut,"%s", str+3 );
				} else
					fprintf(fOut,"%i", (int)(mag*10+0.5) );
				fprintf(fOut,"\n");
				countN++;
			}
		} else {
			if (strncmp(str,"  ---",5)==0)
				weiter = FALSE;
		}
	}
	
	fprintf(fOut,"-------------------------------------------------\n");
	fclose(fOut);	
	fclose(fIn);
	
	fprintf(stderr,"Created chart file %s with %i stars\n", chartname, countN );
}

