
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include "libtl/libtl_string.h"

#include "vs_sub.h"
#include "vs_forms.h"
#include "vs_language.h"
#include "vs_file.h"
#include "vs_extrem.h"
#include "vs_beob.h"
#include "vs_sonne.h"
#include "vs_daten.h"

#ifndef NO_FORMS
FD_gcvs_select *fd_gcvs_select;
FD_star_new    *fd_star_new;
FD_star_select *fd_star_select;
FD_vsnet       *fd_vsnet;
FD_ImportList  *fd_ImportList;
FD_afoev			*fd_afoev;
FD_aavso			*fd_aavso;
#endif


void ca_Daten_Speichern();

/****************************************************************************/
/*  Observers                                                               */
/****************************************************************************/
RObserver* NewObserver()
{
   RObserver *observerNew;
	observerNew = (struct RObserver*) malloc( sizeof(struct RObserver) );
   observerNew->code[0] = '\0';
   observerNew->name[0] = '\0';
   observerNew->email = NULL;
   observerNew->country = NULL;
   observerNew->instruments = NULL;
   observerNew->next = NULL;
   return observerNew;
}

void ReadObservers()
{
   char zeile[256];
   RObserver *observerNew, *observerLast, *observerBefore;
   FILE *f = fopen(pfadObserver,"r");
   observerChange = FALSE;
   if (!f) 
   {
      fprintf(stderr,"Observer-database not found: %s\n",pfadObserver);
      return;
   }
   
   observerFirst = NULL;
   observerLast  = NULL;
   while (fgetline(zeile,sizeof(zeile),f)) 
   {
      observerNew = NewObserver();
      strn0cpy( observerNew->code, zeile, sizeof(observerNew->code) );
      if (fgetline(zeile,sizeof(zeile),f)) 
      {
         strn0cpy( observerNew->name, zeile, sizeof(observerNew->name) );
         if (fgetline(zeile,sizeof(zeile),f))
            SetString( &observerNew->email, zeile, 0 );
         if (fgetline(zeile,sizeof(zeile),f))
            SetString( &observerNew->country, zeile, 0 );
         if (fgetline(zeile,sizeof(zeile),f))
            SetString( &observerNew->instruments, zeile, 0 );
      }
      // sortiert:
      if (observerFirst==NULL)
         observerFirst = observerNew;
      else 
      if (strcmp(observerFirst->code,observerNew->code)>0) 
      {
      	observerNew->next = observerFirst;
      	observerFirst = observerNew;
      } 
      else 
      {
      	bool more = TRUE;
         observerLast = observerFirst;
         observerBefore = NULL;
         while (more) 
         {
         	if (strcmp(observerLast->code,observerNew->code)<0) 
         	{
         		if (observerLast->next==NULL) 
         		{
         			observerLast->next = observerNew;
         			more = FALSE;
         		}
         		else 
         		{
         			observerBefore = observerLast;
	         		observerLast = observerLast->next;
	         	}
	         } 
	         else 
	         {
	         	observerNew->next = observerLast;
	         	observerBefore->next = observerNew;
	         	more = FALSE;
	         }
         }
      }
   }
   fclose(f);
}

void WriteObservers()
{
   RObserver *observerPtr = observerFirst;
   FILE *f = fopen(pfadObserver,"w");
   if (!f)	return;
   
   while (observerPtr) 
   {
      fprintf(f,"%s\n",observerPtr->code);
      fprintf(f,"%s\n",observerPtr->name);
      WritelnString(&observerPtr->email, f );
      DeleteString( &observerPtr->email );
      WritelnString(&observerPtr->country, f );
      DeleteString( &observerPtr->country );
      WritelnString(&observerPtr->instruments, f );
      DeleteString( &observerPtr->instruments );
      observerPtr = observerPtr->next;
   }
   fclose(f);
}

void DeleteObservers()
{
   RObserver *observerPtr = observerFirst, *obsDelete;
   while (observerPtr) 
   {
      DeleteString( &observerPtr->email );
      DeleteString( &observerPtr->country );
      DeleteString( &observerPtr->instruments );
      obsDelete = observerPtr;
      observerPtr = observerPtr->next;
      free( obsDelete );
   }
}

void CheckObserver( const char* obsCode )
{
   RObserver *obsPtr = observerFirst;
   char obsCodeTest[16];
   strn0cpy(obsCodeTest,(char*)obsCode,15);
   toupperstr(obsCodeTest);

   while (obsPtr) 
   {
      if (strcmp(obsCodeTest,obsPtr->code)==0) 
         return;	/* observer exists */
      obsPtr = obsPtr->next;
   }

   obsPtr = NewObserver();
   strcpy( obsPtr->code, obsCodeTest );
   obsPtr->next    = observerFirst;
   observerFirst   = obsPtr;
   observerChange  = TRUE;
   fprintf(stderr,"New observer: %s\n",obsCode);
}


const RObserver* FindObserver( const char* obsCode )
{
   RObserver *obsPtr = observerFirst;
   char obsCodeTest[16];
   strn0cpy(obsCodeTest,(char*)obsCode,15);
   toupperstr(obsCodeTest);

   while (obsPtr) 
   {
      if (strcmp(obsCodeTest,obsPtr->code)==0) 
         return obsPtr;	/* observer exists */
      obsPtr = obsPtr->next;
   }
   return NULL;
}


const int GetObserverIndex( const char* obsCode )
{
	int index = 0;
	RObserver *obsPtr = observerFirst;
	char obsCodeTest[16];
	strn0cpy(obsCodeTest,(char*)obsCode,15);
	toupperstr(obsCodeTest);

	while (obsPtr) 
	{
		if (strcmp(obsCodeTest,obsPtr->code)==0) 
			return index;	/* observer exists */
		obsPtr = obsPtr->next;
		index++;
	}
	return -1;
}


bool CanObserverOutput( const char* obsCode, const struct RHell* hellPtr )
{
	bool isObserver = FALSE;
	if (obsCode[0]==0) 
	{
		isObserver = (hellPtr->art==1) &&
	                (hellPtr->bemerkung[0]!=0) && (hellPtr->bemerkung[0]!='_') &&
                   (hellPtr->bemerkung[0]!=' ');
	}
	else 
	if (hellPtr->observers) 
	{
		char *ptr = strstr(hellPtr->observers,obsCode);
		if (ptr) 
		{
			ptr += strlen(obsCode);
			if (*ptr==' ' || *ptr=='\0')
				isObserver = TRUE;
		}
	}
	return isObserver;
}


int  convertN = 0;
char convertAFOEV[100][5];
char convertAAVSO[100][5];
int  convertStore[100];

int ReadObsCodeConversionList( const char* filename )
{
	char line[128];
	FILE* f = fopen(filename,"r");
	if (f==NULL)	return 0;
	
	convertN = 0;
	while (fgets(line,sizeof(line)-1,f)) 
	{
		int ok;
		char* ptr = strchr(line,'#');
		if (ptr)	*ptr = 0;
		ptr = strchr(line,';');
		if (ptr)	*ptr = 0;
		convertStore[convertN] = 1;
		ok = sscanf(line,"%s %s %i",convertAFOEV[convertN],convertAAVSO[convertN],&convertStore[convertN]);
		if (ok>=2) {
//			fprintf(stderr,"%s --> %s: %i\n",convertAFOEV[convertN],convertAAVSO[convertN],convertStore[convertN]);
			convertN++;		
		}
	}
	
	fclose(f);
	return convertN;
}


/****************************************************************************/
/*  Organisation                                                            */
/****************************************************************************/
ROrganisation* NewOrganisation()
{
   ROrganisation *orgNew;
	orgNew = (struct ROrganisation*) malloc( sizeof(struct ROrganisation) );
   orgNew->code[0] = '\0';
   orgNew->name = NULL;
   return orgNew;
}

void ReadOrganisation()
{
   char zeile[256];
   ROrganisation *orgNew;
   FILE *f = fopen(pfadOrganisation,"r");
   if (!f) 
   {
      fprintf(stderr,"Organisation-database not found: %s\n",pfadOrganisation);
      return;
   }
   
	organisationFirst = NULL;
	organisationBAV   = NULL;
	while (fgetline(zeile,sizeof(zeile),f)) 
	{
		orgNew = NewOrganisation();
		strn0cpy( orgNew->code, zeile, sizeof(orgNew->code) );
		if (fgetline(zeile,sizeof(zeile),f)) 
		{
			SetString( &orgNew->name, zeile, 0 );
		}
		if (strcmp(orgNew->code,"BAV")==0)
			organisationBAV = orgNew;
		orgNew->next = organisationFirst;
		organisationFirst = orgNew;
	}
	fclose(f);
}

void DeleteOrganisation()
{
   ROrganisation *orgPtr = organisationFirst, *orgDelete;
   while (orgPtr) 
   {
      DeleteString( &orgPtr->name );
      orgDelete = orgPtr;
      orgPtr = orgPtr->next;
      free( orgDelete );
   }
}

bool IsObservationInOrg( const struct RHell* aHell, 
                         const struct ROrganisation* anOrg,
                         struct RObserver **findObserverPtr,
                         int *findObserverN )
{
	char str[64];
	struct RObserver *obsPtr = observerFirst;
	const char* obsNameSelect = aHell ? aHell->observers : NULL;
	char* observerNameSelection = NULL;
	int obsN = 0;
		
	if (aHell==NULL)
		return false;
	if (findObserverPtr)
		*findObserverPtr = NULL;
	if (findObserverN)
		*findObserverN = -1;

	if ((aHell->art & 31)!=0) 
	{	// check the "main observer"
  		if (initialAAVSO[0] && strcmp(anOrg->code,"AAVSO")==0)
			return TRUE;
		if (initialBAV[0] && strcmp(anOrg->code,"BAV")==0)
 			return TRUE;
  		if (initialAFOEV[0] && strcmp(anOrg->code,"AFOEV")==0)
 			return TRUE;
 	}

	if (anOrg==NULL)
		return false;
		
	if (obsNameSelect && strlen(obsNameSelect)>0)
	{	// add " " before and after the string for searching purposes
		SetStrings( &observerNameSelection, " ", obsNameSelect );
		AddString( &observerNameSelection, " " );
	}
	else
		return false;

	while (obsPtr) 
	{
		char checkCode[32];
		int i;
  		checkCode[0] = ' ';
		for (i=0; obsPtr->code[i]!=0; i++)
  			checkCode[i+1] = obsPtr->code[i];
  		checkCode[i+1] = ' ';
  		checkCode[i+2] = 0;
  		if (strstr( observerNameSelection, checkCode )!=NULL)
  		{	bool found = false;
			sprintf(str,".%s",anOrg->code);	// e.g. "LTO.AAVSO"
			if (strstr( obsPtr->code, str ))
				found = true;
			if (obsPtr->country && strstr( obsPtr->country, anOrg->code ) != NULL)
				found = true;
			if (found)
			{	if (findObserverPtr)
					*findObserverPtr = obsPtr;
				if (findObserverN)
					*findObserverN = obsN;
				return true;
			}
  		}
  		obsN++;
		obsPtr = obsPtr->next;
	}

	return false;
}


/****************************************************************************/
/* Daten_GCVS                                                               */
/****************************************************************************/

int gcvsUseAll,gcvsUseE,gcvsUseR,gcvsUseU,gcvsUseS,gcvsUseM;
int gcvsMag, gcvsAmplMin, gcvsAmplMax, gcvsDeklMin,gcvsDeklMax;
float gcvsPeriodMin,gcvsPeriodMax;

char* Make_GCVS_Name_From_Filename( const char* starFileName, char* gcvsName )
{
	char gcvs[32];
	int  vn = atoi(starFileName+4);
  	if ((toupper(starFileName[3])=='V') && (vn>99) && (vn<10000))
  	{	if (vn<1000)
	  		sprintf(gcvs,"V0%i ",vn);
	  	else
	  		sprintf(gcvs,"V%i ",vn);
  	}
  	else
  		sprintf(gcvs,"%s     ",starFileName+3);
  	gcvs[6] = '\0';
  	strcat(gcvs,starFileName);
  	gcvs[9] = '\0';
  	toupperstr(gcvs);	
  	return strcpy(gcvsName,gcvs);
}

char* Read_GCVS_Star( const char* name, char* line )
{
	FILE *fGCVS;
	char t[200], suchbst[20];
	bool is2001 = (strstr(pfadGCVS,"iii.dat")!=NULL);	// Format 2001 oder 1989
	int  posSternbild = (is2001 ? 14 : 12);
	int  posName      = (is2001 ?  8 :  7);
	int  lenName      = (is2001 ?  9 :  8);

	sprintf(suchbst,"   ");
	fGCVS = fopen(pfadGCVS,"r");
	do {
		if (fgetline(t,200,fGCVS)!=NULL)
		{
			toupperstr(t);
			if (strncmp(t+posSternbild,suchbst,3)==0)
				strn0cpy(suchbst,t+posSternbild,3);
		}
	} while ((strncmp(t+posName,name,lenName)!=0) && (t[0]>0) && (t[posSternbild]>name[2]));
	fclose(fGCVS);
	
	return strcpy(line,t);
}	

void Daten_GCVS_Suche (const char *nameBild, const char* nameStern)
{
	FILE *fGCVS;
	char name[64], t[200], suchbst[20];
	bool is2001 = (strstr(pfadGCVS,"iii.dat")!=NULL);	// Format 2001 oder 1989

	// den Suchnamen zusammenbasteln
	name[0] = '\0';
	if (toupper(nameStern[0])=='V')
	{
		int varN = atoi(nameStern+1);
		if (varN>99 && varN<1000)
		{
			if (is2001)	// 2001
				sprintf(t,"V%04i ",varN);
			else		// 1989
				sprintf(t,"V %i ",varN);
		}
	}
	strn0cpy(name,nameStern,20);
	while (strlen(name) < (is2001?6:5)) 
		strcat(name," ");
	strcat(name,nameBild);
	toupperstr(name);

	// erst mal fuer Ausgabe 1989
	int posSternbild = 12;
	int posName      =  7;
	int lenName      =  8;
	int posType      = 64;
	int posRect      = 16;
	int posDecl      = 22;
	int posPeriode   = 116;
	int posJD        = 93;
	int posMax       = 74;
	int posMin       = 83;
	int posSpektrum  = 140;
	int posPhase     = 135;
	
	// fuer Ausgabe 2001
	if (is2001) 
	{
		posSternbild = 14;
		posName      =  8;
		lenName      =  9;
		posType      = 53;
		posRect      = 20; 	// 37 fr q.2000
		posDecl      = 28;	// 45 fr q.2000
		posPeriode   = 107;
		posJD        = 88;
		posMax       = 65;
		posMin       = 75;
		posSpektrum  = 134;
		posPhase     = 128;
	}

	if (name==NULL) 
	{	// TL 19.07.&04.08.2001: create complete data set
		diskFirst = NULL;
		diskLoad = malloc(sizeof(struct RDisk));

		fGCVS = fopen(pfadGCVS,"r");
		do {
			if (fgetline(t,200,fGCVS)!=NULL) 
			{
				bool store = TRUE;
				float dekl;
				DatenLoeschen();
				// create star
				goto CreateData;	// bad code, I know
				StoreData:
				{	char stbild[5], stname[20], *ptr;
					strn0cpy( stbild, t+posSternbild, 3 );
					strn0cpy( stname, t+posName, lenName+1 );
					ptr = stname;
					while (*ptr!=' ') ptr++;
					*ptr = 0;
					sprintf(diskLoad->name,"%s%s", stbild, stname);
					tolowerstr(diskLoad->name);
					strn0cpy(diskLoad->stb, stbild,3);
					toupperstr(diskLoad->stb);
					strn0cpy(diskLoad->index, stname,9);
					toupperstr(diskLoad->index);
					sprintf(stern.name,"%s %s",stname,stbild);
				}
				// check type and range
				if (!gcvsUseAll) 
				{
					if (gcvsUseE && stern.gruppe!=STG_BED)			store = FALSE;
					if (gcvsUseR && stern.gruppe!=STG_PULS)		store = FALSE;
					if (gcvsUseU && stern.gruppe!=STG_ERUP)		store = FALSE;
					if (gcvsUseS && stern.gruppe!=STG_IRR)			store = FALSE;
					if (gcvsUseM && stern.gruppe!=STG_MIRA)		store = FALSE;
				}
				if (stern.hellMax > gcvsMag)							store = FALSE; 
				if (stern.hellMin-stern.hellMax < gcvsAmplMin)	store = FALSE;
				if (stern.hellMin-stern.hellMax > gcvsAmplMax)	store = FALSE;
				if (stern.periode < gcvsPeriodMin)					store = FALSE;
				if (stern.periode > gcvsPeriodMax)					store = FALSE;
				dekl = atof(stern.designation+4);
				if (dekl < gcvsDeklMin)									store = FALSE;
				if (dekl > gcvsDeklMax)									store = FALSE;
				// store file
				if (store) {
					fprintf(stderr,"Creating %s\n",stern.name);
					ca_Daten_Speichern();
				}
			}
		} while (t[0]>0);
		fclose(fGCVS);
		fprintf(stderr,"\nCreation of GCVS stars completed. Please restart VARSTARS.\n");
		exit(0);
	}
		
	sprintf(suchbst,"   ");
	fGCVS = fopen(pfadGCVS,"r");
	do {
		if (fgetline(t,200,fGCVS)!=NULL)
		{
			toupperstr(t);
			if (strncmp(t+posSternbild,suchbst,3)==0)
			{
				strn0cpy(suchbst,t+posSternbild,3);
			}
		}
		else
			t[0] = 0;
	} while ((strncmp(t+posName,name,lenName)!=0) && (t[0]>0) && (t[posSternbild]>name[2]));
	fclose(fGCVS);
	
	if (strncmp(t+posName,name,lenName)==0) 
	{
		CreateData:
		strn0cpy(stern.typ,t+posType,9);
		sprintf(stern.designation, "%.4s%.3s*",t+posRect,t+posDecl);
		stern.periode = atof(t+posPeriode);
		stern.gruppe = 0;
		if (stern.periode==0) 
		{
			stern.gruppe = STG_NOVA;
			if (strncmp(stern.typ,"SN",2)==0)      stern.periode = -10;
			else if (strncmp(stern.typ,"NA",2)==0) stern.periode = -50;
			else if (strncmp(stern.typ,"NB",2)==0) stern.periode = -100;
			else if (strncmp(stern.typ,"NC",2)==0) stern.periode = -200;
			else if (strncmp(stern.typ,"ZAND",4)==0) 
			{
				stern.periode = -1;
				stern.gruppe = STG_ERUP;
			}
		}
		stern.max_jd = atof(t+posJD);
		if (stern.max_jd==0) 
		  stern.max_jd = JulDat2000;
		else if (is2001)
			stern.max_jd += 2400000;
		stern.hellMax = (int)floor(atof(t+posMax)*100+0.5);
		stern.hellMin = (int)floor(atof(t+posMin)*100+0.5);
		stern.spektralklasse = GetSpektrumNr(t+posSpektrum);
		stern.pmax= 2;
		stern.pab = 2*atoi(t+posPhase);
		if (stern.pab==0) stern.pab=98;
		stern.pmin = 2;
		stern.pan  = 200-stern.pmax-stern.pab+stern.pmin;

		switch (stern.typ[0]) {
		case 'E':
		  stern.pmin = stern.pab;
		  stern.pmax = stern.pan;
		  stern.pab  = 2;
		  stern.pan  = 2;
		  stern.gruppe = STG_BED;
		  stern.max_jd += stern.periode/2;
		  break;
		case 'R':
			stern.gruppe = STG_PULS;
			break;
		case 'U':
			stern.gruppe = STG_ERUP;
			break;
		case 'S':
			stern.gruppe = STG_IRR;
			break;
		case 'M':
			stern.gruppe = STG_MIRA;
			break;
		}
		flagNeueDaten = TRUE;
		if (name==NULL)
			goto StoreData;
	}
}

void Daten_GCVS ()
{
#ifndef NO_FORMS
	if (pfadGCVS[0]==0) 
		return;
	fl_deactivate_all_forms();
	if (fd_gcvs_select==NULL) 
	{
		fd_gcvs_select = create_form_gcvs_select();
		fl_set_object_label( fd_gcvs_select->LText, LS(104) );
		fl_set_object_label( fd_gcvs_select->LCancel, LS_Cancel );
		fl_set_object_label( fd_gcvs_select->LOK, LS_OK );
		fl_set_object_shortcutkey( fd_gcvs_select->LOK, XK_Return );
	}
	fl_show_form(fd_gcvs_select->gcvs_select,FL_PLACE_MOUSE,FL_FULLBORDER,LS(105));
	fl_raise_form(fd_gcvs_select->gcvs_select);
#endif
}


#ifndef NO_FORMS
void cb_Daten_GCVS (FL_OBJECT *ob, long mode)
{
	char t[100], *ptr;
	bool is2001 = (strstr(pfadGCVS,"iii.dat")!=NULL);	// Format 2001 oder 1989
  
	switch (mode) {
	case 0: /* Cancel */
		fl_hide_form(fd_gcvs_select->gcvs_select);
		fl_activate_all_forms();
		break;
	case 1: /* OK */
		CURSOR_WATCH(fd_gcvs_select->gcvs_select->window);
		XFLUSH;
		strn0cpy( t, fl_get_input(fd_gcvs_select->starname), sizeof(t)-1 );
		ptr = t;
		while (*ptr!=' ' && *ptr!='\0')
			ptr++;
		*ptr = '\0';
		Daten_GCVS_Suche(ptr+1,t);
		CURSOR_NORM(fd_gcvs_select->gcvs_select->window);
		fl_hide_form(fd_gcvs_select->gcvs_select);
		fl_activate_all_forms();
		Daten_Text();
		break;
	}  
}
#endif

void CreateDataOfGCVS( const char* options )
{
	char *ptr;
	float mag = 100, amin = 0, amax = 100, pmin=0, pmax=1000000, dmin=-90, dmax=90;

	ptr = strstr(options,"T=");
	gcvsUseAll = TRUE;
	if (ptr) 
	{
		ptr+=2;
		gcvsUseAll = (strchr(ptr,'a')!=NULL);
		gcvsUseE = (strchr(ptr,'e')!=NULL);
		gcvsUseR = (strchr(ptr,'r')!=NULL);
		gcvsUseU = (strchr(ptr,'u')!=NULL);
		gcvsUseS = (strchr(ptr,'s')!=NULL);
		gcvsUseM = (strchr(ptr,'m')!=NULL);
	}

	ptr = strstr(options,"M=");
	if (ptr) 
	{
		ptr+=2;
		sscanf(ptr,"%f",&mag);
	}
	gcvsMag = (int)(mag*100);
	
	ptr = strstr(options,"A=");
	if (ptr) 
	{
		ptr+=2;
		sscanf(ptr,"%f~%f",&amin,&amax);
	}
	gcvsAmplMin = (int)(amin*100);
	gcvsAmplMax = (int)(amax*100);

	ptr = strstr(options,"P=");
	if (ptr) 
	{
		ptr+=2;
		sscanf(ptr,"%f~%f",&pmin,&pmax);
	}
	gcvsPeriodMin = pmin;
	gcvsPeriodMax = pmax;

	ptr = strstr(options,"D=");
	if (ptr) 
	{
		ptr+=2;
		sscanf(ptr,"%f~%f",&dmin,&dmax);
	}
	gcvsDeklMin = dmin;
	gcvsDeklMax = dmax;

	Daten_GCVS_Suche(NULL,NULL);
}

/****************************************************************************/
/* Daten_Laden                                                              */
/****************************************************************************/
void (*ca_Daten_Laden_Neu)(void);

void Daten_Laden_Neu (char *stb, void (*ca)(void))
{
#ifndef NO_FORMS
	ca_Daten_Laden_Neu = ca; /* call-after-function */
	fl_deactivate_all_forms();
	if (fd_star_new==NULL) 
	{
		fd_star_new = create_form_star_new();
		fl_set_object_label( fd_star_new->LConstellation, LS(100) );
		fl_set_object_label( fd_star_new->LOK, LS_OK );
		fl_set_object_label( fd_star_new->LCancel, LS_Cancel );
		fl_set_object_shortcutkey( fd_star_new->LOK, XK_Return );
	}
	fl_set_input( fd_star_new->Constellation, stb );
	fl_set_input( fd_star_new->Name, "" );
	fl_show_form( fd_star_new->star_new, FL_PLACE_MOUSE, FL_FULLBORDER, LS(102) );
	fl_raise_form( fd_star_new->star_new );
#endif
}

void cb_Daten_Laden_Neu (FL_OBJECT *ob, long mode)
{
#ifndef NO_FORMS
	fl_hide_form( fd_star_new->star_new );
	fl_activate_all_forms();
	if (mode==0)  
	{
		diskLoad = NULL;
		return;
  	}

	Daten_Stern_Neu( fl_get_input( fd_star_new->Constellation ), fl_get_input( fd_star_new->Name ) );

	/* star_select-Form muss neu aufgebaut werden */
	fl_free_form(fd_star_select->star_select);
	fd_star_select = NULL;

	ca_Daten_Laden_Neu();
#endif
}

void Daten_Stern_Neu (const char* sternbild, const char* sternname)
{
	char   stbild[4], stname[32];
	struct RDisk *diskPtr,*diskVor;

	strn0cpy( stbild, sternbild, sizeof(stbild)-1 );
	strn0cpy( stname, sternname, sizeof(stname)-1 );

#ifndef NO_FORMS
	Beob_Close();
#endif
	DatenLoeschen();

	diskLoad = malloc(sizeof(struct RDisk));
	sprintf(diskLoad->name,"%s%s",stbild,stname);
	tolowerstr(diskLoad->name);
	strn0cpy(diskLoad->stb, stbild,3);
	toupperstr(diskLoad->stb);
	strn0cpy(diskLoad->index, stname, sizeof(diskLoad->index)-1 );
	toupperstr(diskLoad->index);
	toupperstr(stname);
	stbild[0] = toupper(stbild[0]);
	sprintf(stern.name,"%s %s",stname,stbild);
  
	/* neuen Stern einsortieren */
	diskVor = diskFirst;
	diskPtr = diskVor->next;
	while (diskPtr!=NULL) 
	{
		if (strcmp(diskPtr->name,diskLoad->name)>0) 
			diskPtr = NULL;
		else 
		{
			diskVor = diskPtr;
			diskPtr = diskPtr->next;
		}
	}
	diskLoad->next = diskVor->next;
	diskVor->next  = diskLoad;
}

char activeSTVFilename[256];
int  activeSTVillegalDate;

FILE *Open_STV (char *stv_name, struct RStern *stern)
{
	FILE*	fDat;
	char 	filename[128], line[128];
	int	i,i1,i2;

	strcpy(activeSTVFilename,stv_name);
	sprintf(filename,"%s%s.stv",pfadSTV,stv_name);
	fDat = fopen(filename,"r");
	if (fDat==NULL) 
	{
		fprintf(stderr,"Open_STV: File not found: '%s'\n",filename);
		return NULL;
	}
	fgetline(stern->name,StrLenName,fDat);
	fgetline(stern->designation,StrLenName,fDat);
	fgetline(stern->karte,StrLenKarte,fDat);
	fgetline(stern->typ,StrLenTyp,fDat);
	fscanf(fDat,"%lf\n",&stern->periode);
	fscanf(fDat,"%lf\n",&stern->max_jd);
	fscanf(fDat,"%i\n",&stern->hellMax);
	fscanf(fDat,"%i\n",&stern->hellMin);
	fscanf(fDat,"%i\n",&stern->pmax);
	fscanf(fDat,"%i\n",&stern->pab);
	fscanf(fDat,"%i\n",&stern->pmin);
	fscanf(fDat,"%i\n",&stern->pan);
	fscanf(fDat,"%i\n",&stern->spektralklasse);
	fscanf(fDat,"%lf\n",&stern->entf);
	i = i1 = i2 = 0;
	fgetline(line,sizeof(line),fDat);
	sscanf(line,"%i %i %i",&i,&i1,&i2);
	stern->HTML = (i!=0);
	stern->gruppe = i1;
	stern->Alert = (i2!=0);
	fscanf(fDat,"%lf\n",&stern->importDeltaT);
	fscanf(fDat,"%i\n",&stern->hellZahl);
	activeSTVillegalDate = 0;
	return fDat;
}

void Read_STV_Hell (FILE *fDat, struct RHell *hellPtr)
{
	char s[256];
	int n = 0;
	fgetline(s,sizeof(s),fDat);
	hellPtr->datum = atof(s);
	if (hellPtr->datum==0)
	{
		activeSTVillegalDate++;
		printf("Read_STV_Hell: Illegal date %i in file '%s'.\n",activeSTVillegalDate,activeSTVFilename);
	}
	fgetline(s,sizeof(s),fDat);
	hellPtr->hell  = atoi(s);
	fgetline(s,sizeof(s),fDat);
	hellPtr->art   = atoi(s);
	sscanf(s,"%i%n",&hellPtr->art,&n);
	hellPtr->observers = NULL;
	if (n>0 && n+1<strlen(s)) 
	{
		while (s[n]==' ' && s[n]!='\0')	n++;
		if (s[n]!='\0')
			hellPtr->observers = NewString( s+n );
	}
	fgetline(hellPtr->bemerkung,StrLenBemerkung,fDat);
}

void Daten_Laden_Stern()
{
	int    			i;
	struct RHell*	hellPtr,*hellNew;
	FILE*				fDat;
 
	DatenLoeschen();
	
	fDat = Open_STV(diskLoad->name,&stern);
	hellFirst  = NULL;
	hellRecent = NULL;
 	for (i=0; i<stern.hellZahl; i++) 
 	{
		hellNew = HellNew();
		if (hellNew==NULL) 
		{
#			ifdef NO_FORMS
			fprintf(stderr,"Wegen Speichermangels knnen die Beobachtungen nicht geladen werden!\n");
#			else
			fl_show_alert("Wegen Speichermangels","knnen die Beobachtungen","nicht geladen werden!",TRUE);
#			endif
			while (hellFirst != NULL) 
			{
 				hellPtr   = hellFirst;
 				hellFirst = hellFirst->next;
 				HellDelete(&hellPtr);
			}
     		stern.hellZahl = 0;
     		hellFirst      = NULL;
   	}
   	else 
   	{
     		if (hellFirst==NULL) 
     			hellFirst = hellNew;
     		else	
       		hellPtr->next = hellNew;
     		Read_STV_Hell(fDat,hellNew);
     		hellNew->next = NULL;
     		hellPtr       = hellNew;
			if (i==stern.hellZahl-100)
				hellRecent = hellNew;
		}
 	}
 	if (hellRecent==NULL)
 		hellRecent = hellFirst;
 	fclose(fDat);

	if (ExtremExists(diskLoad->name))
		ExtremLoad(diskLoad->name);
}

void Daten_Laden_Sonne()
{
	struct RSonne *sonnePtr, *sonneNew;
	int   i;
	FILE *fDat;
	char  name[128];

	sprintf(name,"%s%s.stv",pfadSTV,diskLoad->name);
	fDat = fopen(name,"r");
	if (fDat==NULL) 
	{
		fprintf(stderr,"Open_STV: File not found: '%s'\n",name);
		return;
	}
	fscanf(fDat,"%i\n",&sonneZahl);
	sonneFirst = NULL;
	if (sonneZahl>0) 
	{
		for (i=0; i<sonneZahl; i++) 
		{
			sonneNew = (struct RSonne *) malloc(sizeof(struct RSonne));
			if (i==0)	
				sonneFirst = sonneNew;
			else
				sonnePtr->next = sonneNew;
			sonnePtr = sonneNew;
			fscanf(fDat,"%lf\n",&sonnePtr->datum);
			fscanf(fDat,"%i\n",&sonnePtr->gruppe);
			fscanf(fDat,"%i\n",&sonnePtr->fleck);
			fscanf(fDat,"%i\n",&sonnePtr->seh);
			sonnePtr->next = NULL;
		}
	}
	fscanf(fDat,"%i\n",&sehZahl);
	seh_name[0][0] = '\0';
	for (i=1; i<=sehZahl; i++)
		fgetline(seh_name[i],80,fDat);
	Sun_RelZahl();
	fclose(fDat);
	Daten_Text();
}

int star_select_sternbild = 2;
int star_select_stern     = 2;

#ifndef NO_FORMS

void star_select_browser_stern ()
{
	struct RDisk *diskPtr;
	int i, topline, screenline;
	char stb[4];

	fl_clear_browser(fd_star_select->stern);
	if (star_select_sternbild<2) return;

	strncpy(stb, fl_get_browser_line(fd_star_select->sternbild,
	                                 fl_get_browser(fd_star_select->sternbild)) ,3);
	stb[3]='\0';
	fl_addto_browser(fd_star_select->stern,LS(102));
	i = 1;
	diskPtr = diskFirst->next;
	while (diskPtr!=NULL) 
	{
		if (strcmp(diskPtr->stb,stb)==0) 
		{
			fl_addto_browser(fd_star_select->stern, diskPtr->index);
			i++;
		}
		diskPtr = diskPtr->next;
	}
	if (star_select_stern>i) 
		star_select_stern = i;
	if (star_select_stern<1)
		star_select_stern = 1;  
	fl_select_browser_line(fd_star_select->stern,star_select_stern);
	screenline = fl_get_browser_screenlines(fd_star_select->stern);
	if (star_select_stern<screenline) topline = 0;
	else if (star_select_stern>i-screenline) topline = i-screenline+1;
	else topline = star_select_stern;
	fl_set_browser_topline(fd_star_select->stern,topline);
}

void cb_star_select (FL_OBJECT *ob, long data)
{
	const XEvent *xe;
	static char KEYCHAR[] = "QWERTZUIOP    ASDFGHJKL     YXCVBNM";
	static char stb_char[] = "\0         ";
	static int  focus_on_stern = FALSE;
	int new_stb,new_stern;

	new_stb   = fl_get_browser(fd_star_select->sternbild);
	new_stern = fl_get_browser(fd_star_select->stern);
  
	switch (data) {
	case 0: /* Abbruch */
		fl_hide_form(fd_star_select->star_select);
		fl_activate_all_forms();
		break;
	case 1: /* OK */
	  fl_hide_form(fd_star_select->star_select);
	  fl_activate_all_forms();
	  Beob_Close();
	  DatenLoeschen();
	  star_select_sternbild = new_stb;
	  star_select_stern     = new_stern; 
		if (new_stb==1) 
		{
	    diskLoad = diskFirst;
	    Daten_Laden_Sonne();
	    Sonne_Liste(FALSE);
		} 
		else 
		{
	    char stb[4];
	    strcpy(stb, fl_get_browser_line(fd_star_select->sternbild,new_stb) );
	    besDatumVon = Zeit2JulDat("01.01.1700");
	    besDatumBis = Zeit2JulDat("01.01.2100");
	    if (new_stern==1) {
	      Daten_Laden_Neu(stb,Daten_Text);
	      return;
	    } else {
	      diskLoad = diskFirst->next;
	      while (strcmp(diskLoad->stb,stb)!=0) diskLoad = diskLoad->next;
	      while (new_stern>2) { diskLoad = diskLoad->next; new_stern--; }
	    }  
	    if (diskLoad==NULL)
	      return;
	    Daten_Laden_Stern();
	    Daten_Text();
	  }
	  flagNeueDaten = FALSE;
	  break;
	case 2: /* Sternbild */     
		if (new_stb!=star_select_sternbild) 
		{
			star_select_sternbild = new_stb;
			star_select_browser_stern();
		}
		focus_on_stern = FALSE;
		break;
	case 3: /* Stern */
		focus_on_stern = TRUE;
		break;
	case 9: /* Shortcut */
		xe = fl_last_event();
		if (xe->type==3) 
		{
			char c;
	 		int maxline = fl_get_browser_maxline(fd_star_select->sternbild);
			int i = xe->xkey.keycode-24;
			if (i==-15)
			{	/* Escape */
				stb_char[0] = '\0';
				new_stb = 1;
			} 
			else 
			if (i==-1) 
			{	/* Tab */
				fl_set_focus_object(fd_star_select->star_select,
				                    focus_on_stern ? fd_star_select->sternbild : 
				                    fd_star_select->stern);
				focus_on_stern = !focus_on_stern;
			} 
			else 
			if ((i>=0) && (i<strlen(KEYCHAR))) 
			{ 
	   	c = KEYCHAR[i];
	   	i = strlen(stb_char);
	   	if (i==3) i=0;
		  	stb_char[i] = c;
	  		stb_char[++i] = '\0';
	  		new_stb = 2;
	     while ( (new_stb<=maxline) &&
	             (strncmp(stb_char, fl_get_browser_line(fd_star_select->sternbild,new_stb) ,i)!=0) 
	            ) {
	       new_stb++;    
	     }
	   } 
		else  
	   	new_stb = 0;
		if ((new_stb > 0) && (new_stb < maxline) && (new_stb!=star_select_sternbild)) {
	     star_select_sternbild = new_stb;
				fl_select_browser_line(fd_star_select->sternbild,new_stb);
		    star_select_browser_stern();
			fl_set_browser_topline(fd_star_select->sternbild,new_stb-1);
		  } 
		  if (new_stb==maxline)
		  	stb_char[0] = '\0'; 
		  fl_set_object_label(fd_star_select->stb_string,stb_char);	
	  }  
		break;
	}
}

void Daten_Laden()
{
	struct RDisk *diskPtr;
	char stb[4];

	if (flagNeueDaten)
		Daten_Speichern_Neu();

	fl_deactivate_all_forms();
	if (fd_star_select==NULL) 
	{
		int nr=1;
		int screenline,topline;
		fd_star_select = create_form_star_select();
		fl_set_object_label( fd_star_select->LConstellation, LS(100) );
		fl_set_object_label( fd_star_select->LStar, LS(101) );
		fl_set_object_label( fd_star_select->LCancel, LS_Cancel );
		fl_set_browser_dblclick_callback( fd_star_select->stern, cb_star_select, 1 );
		fl_clear_browser(fd_star_select->sternbild);
		fl_addto_browser(fd_star_select->sternbild,"Sonne");
		stb[0] = '\0';
		diskPtr = diskFirst->next;
		while (diskPtr!=NULL) 
		{
			if (strcmp(stb,diskPtr->stb)!=0) 
			{
	   		strcpy(stb,diskPtr->stb);
	   		fl_addto_browser(fd_star_select->sternbild,stb);
	   		nr++;
	   	}
			diskPtr = diskPtr->next;
		}
		if (nr<star_select_sternbild) star_select_sternbild = nr;
		fl_select_browser_line(fd_star_select->sternbild,star_select_sternbild);
		screenline = fl_get_browser_screenlines(fd_star_select->sternbild);
		if (star_select_sternbild<screenline) topline = 0;
		else if (star_select_sternbild>nr-screenline) topline = nr-screenline+1;
		else topline = star_select_sternbild;
		fl_set_browser_topline(fd_star_select->sternbild,topline);
		fl_set_object_shortcut(fd_star_select->shortcut, "abcdefghijklm" ,FALSE);
		fl_set_object_shortcut(fd_star_select->shortcut1,"nopqrstuvwxyz^[^i" ,FALSE);
		fl_set_object_shortcutkey( fd_star_select->shortcut, XK_Tab );
		fl_set_object_shortcutkey( fd_star_select->ok, XK_Return );
	}

	star_select_browser_stern();
	fl_show_form(fd_star_select->star_select,FL_PLACE_MOUSE,FL_FULLBORDER,"Select Star");
}

#endif // !NO_FORMS


struct RDisk* Stern_Laden( const char* theName )
{
	struct RDisk *diskTest;
	char sternname[StrLenName];
	strn0cpy(sternname,theName,StrLenName-1);
	tolowerstr(sternname);
	diskTest = diskFirst;
	while ((diskTest!=NULL) && (strcmp(diskTest->name,sternname)!=0))
		diskTest = diskTest->next;
	if (diskTest==NULL)
	{
		return NULL;
	}
	else 
	{
		diskLoad = diskTest;
		if (diskLoad==diskFirst)
			Daten_Laden_Sonne();
		else
			Daten_Laden_Stern();
	}	
	return diskLoad;
}


/****************************************************************************/
/* Daten_Speichern                                                          */
/****************************************************************************/


void Daten_Speichern_Neu()
{
#ifndef NO_FORMS
	int q;
	fl_set_goodies_font(FL_BOLD_STYLE,FL_LARGE_SIZE);
	q = fl_show_question(LS(103),1);
	if (q==1)
		Daten_Speichern();
#endif
}

void Daten_Speichern()
{
	if (diskLoad==NULL) 
		Daten_Laden_Neu("",ca_Daten_Speichern);
	else
		ca_Daten_Speichern();
}

void ca_Daten_Speichern()
{
	int  				i;
	char 				datname[128];
	struct RHell*	hellPtr;
	struct RSonne*	sonnePtr;
	FILE*				fDat;

	// Sicherheitstest, TL 22.02.03
	hellPtr = hellFirst;
	while (hellPtr!=NULL) 
	{
		if (hellPtr->datum==0)
		{
#			ifdef NO_FORMS
			fprintf(stderr,"Illegales Datum! Kann das File nicht speichern!\n");
#			else
			fl_show_alert("Illegales Datum !","Kann das File","nicht speichern!",TRUE);
#			endif
			return;
		}
		hellPtr = hellPtr->next;
	}

	sprintf(datname,"%s%s.stv",pfadSTV,diskLoad->name);
	fDat = fopen(datname,"w");
	if (fDat==NULL) 
	{
		fprintf(stderr,LS(110),datname);
		return;
	}
	if (diskLoad!=diskFirst) 
	{
		fprintf(fDat,"%s\n",stern.name);
		fprintf(fDat,"%s\n",stern.designation);
		fprintf(fDat,"%s\n",stern.karte);
		fprintf(fDat,"%s\n",stern.typ);
		fprintf(fDat,"%f\n",stern.periode);
		fprintf(fDat,"%f\n",stern.max_jd);
		fprintf(fDat,"%i\n",stern.hellMax);
		fprintf(fDat,"%i\n",stern.hellMin);
		fprintf(fDat,"%i\n",stern.pmax);
		fprintf(fDat,"%i\n",stern.pab);
		fprintf(fDat,"%i\n",stern.pmin);
		fprintf(fDat,"%i\n",stern.pan);
		fprintf(fDat,"%i\n",stern.spektralklasse);
		fprintf(fDat,"%f\n",stern.entf);
		fprintf(fDat,"%c %i %c\n", stern.HTML?'1':'0', stern.gruppe, stern.Alert?'1':'0' );
		fprintf(fDat,"%f\n",stern.importDeltaT);
		fprintf(fDat,"%i\n",stern.hellZahl);
		hellPtr = hellFirst;
		while (hellPtr!=NULL) 
		{
			fprintf(fDat,"%f\n",hellPtr->datum);
			fprintf(fDat,"%i\n",hellPtr->hell);
			fprintf(fDat,"%i",hellPtr->art);
			if (hellPtr->observers)
				fprintf(fDat," %s",hellPtr->observers);
			fprintf(fDat,"\n");
			fprintf(fDat,"%s\n",hellPtr->bemerkung);
			hellPtr = hellPtr->next;
		}
	}
	else 
	{
		fprintf(fDat,"%i\n",sonneZahl);
		sonnePtr = sonneFirst;
		while (sonnePtr!=NULL) 
		{
			fprintf(fDat,"%f\n",sonnePtr->datum);
			fprintf(fDat,"%i\n",sonnePtr->gruppe);
			fprintf(fDat,"%i\n",sonnePtr->fleck);
			fprintf(fDat,"%i\n",sonnePtr->seh);
			sonnePtr = sonnePtr->next;
		}
		fprintf(fDat,"%i\n",sehZahl);
		for (i=1; i<=sehZahl; i++)
			fprintf(fDat,"%s\n",seh_name[i]);
	}
	fclose(fDat);
	flagNeueDaten = FALSE;
}

/****************************************************************************/
/* ImportList                                                               */
/****************************************************************************/

int importListNumber = 0;

void OpenImportList( const char* theTitle )
{
#ifndef NO_FORMS
  	if (fd_ImportList==NULL) 
	{
 		fd_ImportList = create_form_ImportList();
		fl_set_object_label( fd_ImportList->Sort, LS(117) );
		fl_set_object_label( fd_ImportList->Close, LS_Close );
 	}
 	fl_clear_browser(fd_ImportList->List);
 	importListNumber = 0;
 	fl_set_object_label( fd_ImportList->Number, "0" );
 	fl_show_form(fd_ImportList->ImportList,FL_PLACE_MOUSE,FL_FULLBORDER,theTitle);
  	CURSOR_WATCH(fd_ImportList->ImportList->window);
#endif
}

void ImportListAdd( const char* theLine )
{
#ifndef NO_FORMS
	char str[10];
	fl_addto_browser( fd_ImportList->List, theLine );
	importListNumber++;
	sprintf(str,"%i",importListNumber);
 	fl_set_object_label( fd_ImportList->Number, str );
	XFLUSH;
#endif
}

#ifndef NO_FORMS

void cb_ImportListSort (FL_OBJECT *ob, long mode)
{
# define LINESIZE 128
	int   maxline, i,j;
	char *lines,swapline[LINESIZE];
		
 	fl_deactivate_all_forms();
	CURSOR_WATCH(fd_ImportList->ImportList->window);

	maxline = fl_get_browser_maxline(fd_ImportList->List);
	lines = malloc(LINESIZE*maxline);
	
	for (i=0; i<maxline; i++)
		strcpy(&lines[i*LINESIZE], fl_get_browser_line(fd_ImportList->List,i+1) );
		
	for (i=0; i<maxline-1; i++)
		for (j=i+1; j<maxline; j++)
			if (strcmp(&lines[i*LINESIZE],&lines[j*LINESIZE])>0) 
			{
				strcpy(swapline,&lines[i*LINESIZE]);
				strcpy(&lines[i*LINESIZE],&lines[j*LINESIZE]);
				strcpy(&lines[j*LINESIZE],swapline);
			}

	fl_clear_browser(fd_ImportList->List);
	
	for (i=0; i<maxline; i++)
		fl_addto_browser(fd_ImportList->List, &lines[i*LINESIZE]);

   CURSOR_NORM(fd_ImportList->ImportList->window);
   fl_activate_all_forms();
#	undef LINESIZE
}

void cb_ImportListClose (FL_OBJECT *ob, long mode)
{
	fl_hide_form(fd_ImportList->ImportList);
}

#endif

/****************************************************************************/
/* Beob_VSNET                                                               */
/****************************************************************************/

void Beob_VSNET ()
{
	char text[32];

	if (flagNeueDaten)
		Daten_Speichern_Neu();

#ifndef NO_FORMS
	fl_deactivate_all_forms();
	if (fd_vsnet==NULL) 
	{
		fd_vsnet = create_form_vsnet();
		fl_set_object_label( fd_vsnet->LFile, LS(116) );
		fl_set_object_label( fd_vsnet->LAvg, LS(114) );
		fl_set_object_label( fd_vsnet->LObserver, LS(118) );
		fl_set_object_label( fd_vsnet->LScale, LS(83) );
		fl_set_object_label( fd_vsnet->LRename, LS(280) );
		fl_set_button( fd_vsnet->LRename, 1 );
		fl_set_object_label( fd_vsnet->LRescan, LS(281) );
		fl_set_object_label( fd_vsnet->OK, LS_OK );
		fl_set_object_label( fd_vsnet->Cancel, LS_Cancel );
		fl_set_object_shortcutkey( fd_vsnet->OK, XK_Return );
	}
	fl_set_input(fd_vsnet->File, pfadVSNET);
	sprintf(text,"%f",importDeltaT);
	fl_set_input( fd_vsnet->Days, text );
	fl_set_input( fd_vsnet->Observer, "" );
	fl_show_form( fd_vsnet->vsnet, FL_PLACE_MOUSE, FL_FULLBORDER, LS(115) );
#endif
}

void cb_Beob_VSNET (FL_OBJECT *ob, long mode)
{
	char   			zeile[128], str[128], *ptr, filename[128], observerCode[20], onlyScanName[20];
	struct RDisk  	*diskPtr,*diskLoadOld;
	struct RAlias 	*alias;
	struct RHell	*hellPtr, hellOld;
	FILE* fDat;
	double 		tag, JDtoday,JDearly;
	int    		p,pd,ph,len;
	TTimeZone 	old_JulDatZone = JulDatZone;
	bool			doRenameFile = FALSE;

	observerCode[0] = '\0';
	onlyScanName[0] = '\0';
	if (scriptRun) 
	{
		fDat = fopen(pfadVSNET,"r");
  		if (fDat==NULL) {
  	  		fprintf(stderr,"VSNET-Data-File '%s' not found\n",pfadVSNET);
  	  		return;
  		}
  	
  		fprintf(stderr,"VSNET-Data-File '%s'\n",pfadVSNET); 
		strcpy(filename,pfadVSNET);
		
		if (*scriptArg!='\0')
			importDeltaT = atof(scriptArg);
		fprintf(stderr," Range of days: %f\n",importDeltaT);	

	}
	else 
	{
#ifndef NO_FORMS
	  	fl_hide_form(fd_vsnet->vsnet);
  		if (mode==0) 
  		{	/* Cancel */
	  		fl_activate_all_forms();  
			return;
  		}

		strcpy(filename,fl_get_input(fd_vsnet->File));
  		if (filename[0]=='~') 
  		{
  			strcpy(zeile,filename+1);
  			strcpy(filename,pfadHome);
  			strcat(filename,"/");
  			strcat(filename,zeile);
  		}
  		fDat = fopen(filename,"r");
  		if (fDat==NULL) 
		{
 			fl_activate_all_forms();
  	  		return;
  		}

		OpenImportList("VSNET Data");
  		importDeltaT = atof( fl_get_input(fd_vsnet->Days) );
  		strncpy( observerCode, fl_get_input(fd_vsnet->Observer), sizeof(observerCode) );
  		observerCode[sizeof(observerCode)-1] = '\0';
  		doRenameFile = fl_get_button( fd_vsnet->LRename );
  		strncpy( onlyScanName, fl_get_input(fd_vsnet->RescanStar), sizeof(onlyScanName) );
  		onlyScanName[sizeof(onlyScanName)-1] = '\0';
  		toupperstr(onlyScanName);
#endif
	}

	JulDatZone = _UT;
 	diskLoadOld = diskLoad;
  	hellOld.datum = 0;
  	
	/* heutiges JD holen */
	JDtoday = Zeit2JulDat( GetDateStr(zeile) ) +1;
	JDearly = Zeit2JulDat("01.01.1800");

	/* Daten durchgehen */
	while (fgetline(zeile,128,fDat)!=NULL)
	{
		/* Kommentarzeile berlesen */
		ptr = strchr(zeile,'%');
		if ((ptr!=NULL) && (ptr-zeile<5))
			zeile[0] = '\0';
		 	
		/* im AAVSO-Format die Stern-Position lschen */
		pd = 0;
		for (p=0; p<7; p++)
			if (isdigit(zeile[p])) pd++;
		if (((zeile[4]=='+') || (zeile[4]=='-')) && (pd==6))
			deletestr(zeile,0,8);
		 	
		/* "," zu "." und Klein- zu Grobuchstaben und Tabulator zu Space */
		for (p=0; p<strlen(zeile); p++) 
		{
		    	if (zeile[p]==',') 
		      		zeile[p] = '.';
		    	else if (zeile[p]=='\t')
		      		zeile[p] = ' ';
		    	else
		      		zeile[p] = toupper(zeile[p]);
		}
		memmove(zeile+1,zeile,127);
		zeile[0] = ' ';

		/* Aliaswandlung */
		alias = aliasFirst;
		while (alias!=NULL) 
		{
		       ptr = strstr(zeile,alias->muster);
		       if (ptr!=NULL && isspace(*(ptr+strlen(alias->muster))) ) 
		       {	// Ersetzung
		          char s[128];
		          *ptr = (char)0;
		          sprintf(s,"%s%s%s", zeile, alias->org, ptr+strlen(alias->muster));
		          strcpy(zeile,s);
		       }
		       alias = alias->next; 
		    }
		 
		    /* NOVAx zu Nx krzen */
		    ptr = strstr(zeile,"NOVA");
		    if (ptr!=NULL) {
		       deletestr(ptr,1,3);
		       if (ptr[1]==' ') 
		          deletestr(ptr,1,1);
		       sscanf(ptr+1,"%i",&len);
		       /* if ((len>1900) && (len<2100)) deletestr(ptr,1,2); raus 1.1.99 */
		    }
		 
		    /* "N Stb xx" nach "StbNxx" ndern */
		    ptr = strstr(zeile,"N ");
		    if (ptr!=0) {
		       if (isupper(ptr[2]) && isupper(ptr[3])
		          && isupper(ptr[4]) && (ptr[5]==' ')
		          && isdigit(ptr[6]) && isdigit(ptr[7])) 
		       {
		          char s[128];
		          ptr[0] = (char)0;
		          ptr[5] = (char)0;
		          sprintf(s,"%s%sN%s", zeile, ptr+2, zeile+6);
		          strcpy(zeile,s);
		       }
		    }
		//fprintf(stderr,"%s\n",zeile);

		    /* "SN" nach "_SN" */
		    ptr = strstr(zeile," SN");
		    if (ptr!=0) {
		    	int year = atoi(ptr+2);
		    	if (year>1900 && year<2100 && isalpha(ptr+6))
		       {
		       	insertstr("_",zeile,ptr-zeile);
		       }
		    }
		    
		    /* aktuelle Zeile nach gltigem Sternnamen durchsuchen */
		    len = strlen(zeile);
		    diskLoad = NULL;
		    diskPtr  = diskFirst->next;
		    while (diskPtr!=NULL) {
		       sprintf(str," %s%s ",diskPtr->stb,diskPtr->index);
		       ptr = strstr(zeile,str);
		       if (ptr==NULL) {
		          sprintf(str," %s %s ",diskPtr->stb,diskPtr->index);
		          ptr = strstr(zeile,str);
		       }
		       if (ptr==NULL) {
		          sprintf(str," %s %s ",diskPtr->index,diskPtr->stb);
		          ptr = strstr(zeile,str);
		       }
		       if (ptr && onlyScanName[0]) {	// TL 08.10.01
		       	//printf("%s\n%s\n%i\n\n",onlyScanName,ptr,strncmp(ptr+1,onlyScanName,strlen(onlyScanName)) );
		       	if (strncmp(ptr+1,onlyScanName,strlen(onlyScanName))!=0)
		       		ptr = NULL;
		       }
		       if (ptr!=NULL) { /* Name gefunden an Position ptr+1 */
		          /* p = Position des ersten Zeichens des Namens */
		          p = ptr-zeile+1;
		          /* ph = erstes Nicht-Leerzeichen nach dem Namen */
		          ph = p+strlen(str)-1;
		          while ((zeile[ph]==' ') && (ph<len)) ph++;
		          /* Im folgenden wird
		             pd = Position des Datums
		             ph = Position der Helligkeit
		          */
		          tag = atof(zeile);
		          if ((p>11) && (tag>18000101) && (tag<21000101))
		          pd=1; /* Datum (8 Tagesstellen) steht am Zeilenanfang */
		          else if ((p>5) && (tag>0) && (tag<32) && (strchr(zeile,'.')-zeile<6)) {
		          /* Datum (1 oder 2 Tagesstellen) steht am Anfang
		             und wird auf 8 Stellen expandiert 
		          */
		       	if (zeile[2]=='.') {
		             insertstr("0",zeile,pd);
		             len++;
		             p++;
		             ph++;
		          }
		          GetDateStr(str);
				str[2]='\0';
				str[5]='\0';
				str[10]='\0';
				if (tag>atof(str)+1) 
					sprintf(str+20,"%s%02i", str+6, atoi(str+3)-1);
				else
					sprintf(str+20,"%s%s", str+6, str+3);
				strcpy(str,str+20);
				pd = 1;
				insertstr(str,zeile,pd);
				len+=6;
				p  +=6;
				ph +=6;
			} else { /* das Datum steht nach dem Namen */
				pd = ph;
				tag = atof(zeile+pd);
				while ((zeile[ph]!='.') && (zeile[ph]!=' ') && (ph<len)) ph++;
				if (ph<len) 
					switch (ph-pd) { /* = Anzahl der Vorkommastellen */
					case 1:
					case 2:
						if ((tag>0) && (tag<32))  {
							if (ph-pd==1) {
								insertstr("0",zeile,pd);
								len++;
							}
		            GetDateStr(str);
		            str[2]='\0';
		            str[5]='\0';
		            str[10]='\0';
		            if (tag>atof(str)+1) 
		              sprintf(str+20,"%s%02i", str+6, atoi(str+3)-1);
		            else
		              sprintf(str+20,"%s%s", str+6, str+3);
		            insertstr(str+20,zeile,pd);
		            len+=6;
		          }
		          else 
		            p=0;
		          break;
		        case 4:
		          GetDateStr(str);
		          if ((strncmp(str,"01.01",5)==0) && (strncmp(zeile+pd,"12",2)==0))
		            sprintf(str,"%i",atoi(str+6)-1);
		          else 
		            strcpy(str,str+6);
		          insertstr(str,zeile,pd);
		          len+=4;
		          break;
		        case 5:
		        	insertstr("J4",zeile,pd); /* fr 24xxxxx */
		        	len+=2;
		        	break;
		        case 6:
		          if (tag>900101) 
		            insertstr("19",zeile,pd);
		          else 
		            insertstr("20",zeile,pd);
		          len+=2;
		          break;
		        case 7:
		          if ((tag<2000000) || (tag>2999999))
		            p=0;
		          else 
		            zeile[pd]='J';
		          break;
		        case 8:
		          if ((tag<18000101) || (tag>21000101))
		            p=0;
		          break;
		        default:
		          p=0;
		        }
		        while ((zeile[ph]!=' ') && (ph<len)) ph++;
		        while ((zeile[ph]==' ') && (ph<len)) ph++;
		      }
			if ((p>=0) && (ph<len)) {
				if (zeile[ph]=='[') zeile[ph]='<';
				if (zeile[ph]=='<') ph++;
				if (atof(zeile+ph)==0) p=0;
			} else {
				p=0;
			}
		    } else
			p = 0;
			if (p==0)
				diskPtr = diskPtr->next;
			else {
				diskLoad= diskPtr;
				diskPtr = NULL;
			}
		}
		if (diskLoad!=NULL) {
			bool ok = TRUE;
      	DatenLoeschen();
      	Daten_Laden_Stern();
      	hellPtr = HellNew();
      	/* Datum scannen und testen */
      	if (zeile[pd]=='J')
        		hellPtr->datum = 2000000+atof(zeile+pd+1);
      	else
	{
        		sprintf(str,"00.%.2s.%.4s", zeile+pd+4, zeile+pd);
        		hellPtr->datum = Zeit2JulDat(str) + atof(zeile+pd+6);
      	}
      	if ((hellPtr->datum<JDearly) || (hellPtr->datum>JDtoday)) 
	{
      		fprintf(stderr,"Wrong date: %s\n",zeile);
      		HellDelete(&hellPtr);
      		ok = FALSE;
      	}
     		/* Helligkeit scannen und interpretieren */
     		if (ok) 
		{
		      hellPtr->hell = (int)floor(atof(zeile+ph)*100+0.5);
		      if (zeile[ph]!='.' && zeile[ph+1]!='.' && zeile[ph+2]!='.') //(hellPtr->hell>2000)
					hellPtr->hell = hellPtr->hell / 10;
		     	if (zeile[ph-1]=='<')
					hellPtr->hell += 10000;
		     	if (hellPtr->hell > 20000) {
		     		HellDelete(&hellPtr);
		     		ok = FALSE;
		     	}
		   }
			if (ok) {	// Daten sind ok und werden aufgenommen      	
			char logline[128];
			char observer[64]; observer[0] = '\0';
         	sprintf(logline,"@f%-3.3s %-7.7s %s  ", 
                            diskLoad->stb, diskLoad->index, ZeitDatum(hellPtr->datum,str) );
         	if (hellPtr->hell>9999)
           		sprintf(str, "<%5.2f",hellPtr->hell*0.01-100);
         	else 
           		sprintf(str, " %5.2f",hellPtr->hell*0.01);
				strcat(logline,str);
				/* Observer-Code suchen */
				while (zeile[ph]!=' ' && zeile[ph]!='\0')	ph++;
				while (zeile[ph]==' ' && zeile[ph]!='\0')	ph++;
				if (isalpha(zeile[ph]) && isalpha(zeile[ph+1]) && isalpha(zeile[ph+2])) 
				{
					if (!isalpha(zeile[ph+3]) || zeile[ph+3]=='.') 
					{
						int len = 3;
						while (zeile[ph+len]!=' ' && zeile[ph+len]!='\0' && zeile[ph+len]!='=' && zeile[ph+len]!='<' && zeile[ph+len]!='&')
							len++;
						strn0cpy(observer,zeile+ph,len);
					}
				}
				if (observer[0]==0) 
				{
					strcpy(observer,observerCode);
				}
				if (observer[0]!=0) 
				{
					strcat(logline,"   ");
					strcat(logline,observer);
				}
         	/* Ergebnis ausgeben */
         	if (scriptRun)
           		fprintf(stderr,"%s\n",logline+2);
         	else 
         		ImportListAdd( logline );
         	if ((hellOld.datum==hellPtr->datum) && (hellOld.hell==hellPtr->hell))
   				HellDelete(&hellPtr);
         	else
         	{	/* neue Daten in Beobachtungsliste eintragen */
   				float deltaT = importDeltaT;
   				if (stern.importDeltaT>=0)
   					deltaT = stern.importDeltaT;
               if (observer[0]!='\0') 
		{
                  AddStrings( &hellPtr->observers, " ", observer );
                  CheckObserver( observer );
               }
               if (!canAverageBAVobs
               && IsObservationInOrg(hellPtr,organisationBAV,NULL,NULL))
               	deltaT = 0;	// never average BAV observations
   				hellPtr = HellPtrAdd(hellPtr,deltaT);
               if (observer[0]!='\0') 
		{
                  if (HellPtrDoppelTest(hellPtr,deltaT))
                  	HellPtrDelete(&hellPtr);
               }
					if (hellPtr)
						Daten_Speichern();
				}
			}
		}
	}
	fclose(fDat);

	if (doRenameFile) 
	{
		sprintf(zeile,"%s.scanned",filename);
		rename(filename,zeile);
	}
	
  	JulDatZone = old_JulDatZone;
  	if (!scriptRun) 
  	{
#		ifndef NO_FORMS
		CURSOR_NORM(fd_ImportList->ImportList->window);
		fl_activate_all_forms();
#		endif
	}

  	DatenLoeschen();
  	diskLoad = diskLoadOld;
  	if ((diskLoad!=NULL) && (diskLoad!=diskFirst)) 
  	{
		Daten_Laden_Stern();
		Daten_Text();
	}  
}

/****************************************************************************/
/* Beob_AFOEV                                                              */
/****************************************************************************/

#ifndef NO_FORMS

void Beob_AFOEV ()
{
	if (flagNeueDaten)
		Daten_Speichern_Neu();

	fl_deactivate_all_forms();
	if (fd_afoev==NULL) 
	{
		char str[32];
		fd_afoev = create_form_afoev();
		fl_set_input(fd_afoev->lineFirst, "0");
		fl_set_input(fd_afoev->lineLast, "0");
		fl_set_input(fd_afoev->timeAvg, "0");
		fl_set_input(fd_afoev->fileName, pfadStart);
		fl_set_object_label( fd_afoev->LFirst, LS(112) );
		fl_set_object_label( fd_afoev->LLast, LS(113) );
		fl_set_object_label( fd_afoev->LAverage, LS(114) );
		fl_set_object_label( fd_afoev->LScale, LS(83) );
		fl_set_object_label( fd_afoev->LObserver, LS(120) );
		fl_set_object_label( fd_afoev->Cancel, LS_Cancel );
		fl_set_object_shortcutkey( fd_afoev->OK, XK_Return );
		sprintf(str,"%f",importDeltaT);
		fl_set_input( fd_afoev->timeAvg, str );
		fl_addto_choice( fd_afoev->fileType,"This star");
		fl_addto_choice( fd_afoev->fileType,"Observer");
		fl_set_choice( fd_afoev->fileType, 2 );	// activate observer
	}
	fl_show_form( fd_afoev->afoev, FL_PLACE_MOUSE, FL_FULLBORDER, LS(111) );
}

void cb_AFOEV (FL_OBJECT *ob, long mode)
{
	long   	count1,count2;
	float  	countAvg;
	int  	fileType;	// 0= obs. of star, 1= obs. of observer
	char  	filename[256], zeile[256], log[256], observer[StrLenObsCode], obsCode[StrLenObsCode], starName[StrLenName];
	FILE		*fDat, *fMiss = NULL, *fScanned = NULL;
	struct RHell*	hellPtr;
	TTimeZone	old_JulDatZone = JulDatZone;
	const int posStar[2] = { -1, 0 };
	const int posDate[2] = { 3, 9 };
	const int posMag[2] = { 15, 21 };
	const int posObs[2] = { 0, -1 };

	count1 = atoi(fl_get_input(fd_afoev->lineFirst)) -1;
	count2 = atoi(fl_get_input(fd_afoev->lineLast));
	if (count2==0) 
		count2 = 100000;
	countAvg = atof(fl_get_input(fd_afoev->timeAvg));
	strcpy(obsCode,fl_get_input(fd_afoev->obsCode));
  	strcpy(filename,fl_get_input(fd_afoev->fileName));
	if (filename[0]=='~') 
	{
		strcpy(zeile,filename+1);
		strcpy(filename,pfadHome);
		strcat(filename,"/");
		strcat(filename,zeile);
	}
	fileType = fl_get_choice( fd_afoev->fileType ) -1;
	fl_hide_form(fd_afoev->afoev);

	fDat = fopen(filename,"r");
  	if (fDat==NULL)  
	{
		mode = 0;
  	}
 	if (mode==0) 
	{	/* Cancel */
 		if (fDat)
	 		fclose(fDat);
  		fl_activate_all_forms();
		return;
 	}  

	OpenImportList("AFOEV Data");
		
	switch (fileType)
	{
	case 0:
		sprintf(observer,"123.AFOEV");
		break;
	case 1:
		strcpy( observer, obsCode );
		CheckObserver( observer );
		sprintf(filename,"%s/afoev_scan_miss.lst",pfadHome);
		fMiss = fopen(filename,"w");
		sprintf(filename,"%s/afoev_scan.lst",pfadHome);
		fScanned = fopen(filename,"w");
		break;
	}

	JulDatZone = _UT;
 	/* Daten durchgehen */
	while ((count1>0) && (fgetline(zeile,64,fDat)!=NULL)) count1--;

	while ((count2>0) && (fgetline(zeile,64,fDat)!=NULL)) 
	{
		struct RDisk *testLoad = NULL;
		count2--;
		hellPtr = HellNew();
		hellPtr->datum = 2400000 + atof(zeile+posDate[fileType]);
		if (zeile[posMag[fileType]]==' ') zeile[posMag[fileType]] = '0';
			hellPtr->hell = atof(zeile+posMag[fileType])*100.0 + 0.5;
		if (zeile[posMag[fileType]-1]=='<')
			hellPtr->hell += 10000;

		switch (fileType)
		{
		case 0:
			strncpy( observer, zeile+posObs[fileType], 3 );
			CheckObserver( observer );
			testLoad = diskLoad;
			break;
		case 1:
			strncpy( starName, zeile+posStar[fileType], 10 );
			{	char *ptr = starName;
				while (*ptr!=' ')	ptr++;
				*ptr = '\0';
			}
			DatenLoeschen();
			testLoad = Stern_Laden(starName);
			break;
		}
		
		if (testLoad)
		{
	    		sprintf(log, "@f%-3.3s %-5.5s  %s  ", diskLoad->stb, diskLoad->index, 
			        ZeitDatum(hellPtr->datum,filename) );
			if (hellPtr->hell>9999)
				sprintf(filename, "<%5.2f",hellPtr->hell*0.01-100);
			else 
				sprintf(filename, " %5.2f",hellPtr->hell*0.01);
			strcat(log,filename);
			strcat(log,"   ");
			strcat(log,observer);

			hellPtr->bemerkung[0] = '\0';
			hellPtr->art          = 0;

			switch (fileType)
			{
			case 0:
				hellPtr = HellPtrAdd(hellPtr,countAvg);
				AddStrings( &hellPtr->observers, " ", observer );
				break;
			case 1:
				/* Falsche Eintraege korrigieren, TL 14.10.04
				if (HellPtrTest(hellPtr,"HBA",0.1))
				{
					struct RHell* hp = HellPtrTest(hellPtr,"HBA",0.1);
					if (hp->hell == hellPtr->hell)
					{
						SetString( &hp->observers, "HBL", 0 );
						strcat(log,"  AUSTAUSCH");
						Daten_Speichern();
					}
				}
				else
				*/
				if (HellPtrTest(hellPtr,observer,0.1))
				{	// Eintrag schon vorhanden
					strcat(log,"  Doppeleintrag");
				}
				else
				{
					hellPtr = HellPtrAdd(hellPtr,countAvg);
					AddStrings( &hellPtr->observers, " ", observer );
					Daten_Speichern();
				}
				break;
			}

			if (fScanned)
				fprintf(fScanned,"%s\n",log+2);
			else
				ImportListAdd( log );
		}
		else
		{
			sprintf(zeile,"@f%s missing...",starName);
			ImportListAdd( zeile );
			if (fMiss)
				fprintf(fMiss,"%s\n",zeile+2);
		}
	}
  	
	fclose(fDat);
	if (fMiss)
		fclose(fMiss);
	if (fScanned)
		fclose(fScanned);

	/* Daten_Speichern(); */
	JulDatZone = old_JulDatZone;
	CURSOR_NORM(fd_ImportList->ImportList->window);
	fl_activate_all_forms();
	Daten_Text();
}

#endif

/****************************************************************************/
/* Beob_AAVSO                                                              */
/****************************************************************************/

#ifndef NO_FORMS

void Beob_AAVSO ()
{
	if (flagNeueDaten)
		Daten_Speichern_Neu();

	fl_deactivate_all_forms();
	if (fd_aavso==NULL) 
	{
		char str[256];
		fd_aavso = create_form_aavso();
		sprintf(str,"%s/",pfadHome);
		fl_set_input( fd_aavso->FileName, str );
		fl_set_object_label( fd_aavso->LAverage, LS_Average );
		fl_set_object_label( fd_aavso->LDays, LS_Days );
		fl_set_object_label( fd_aavso->LFiletype, LS(231) );
		fl_addto_choice( fd_aavso->FileType,"AAVSO raw");
		fl_addto_choice( fd_aavso->FileType,"AAVSO sum");
		fl_addto_choice( fd_aavso->FileType,"KSOLOREP");
		fl_addto_choice( fd_aavso->FileType,LS(232));
		fl_set_choice( fd_aavso->FileType, 2 );	// activate AAVSO sum
		fl_set_object_label( fd_aavso->Ok, LS_OK );
		fl_set_object_shortcutkey( fd_aavso->Ok, XK_Return );
		fl_set_object_label( fd_aavso->Cancel, LS_Cancel );
		//sprintf(str,"%f",importDeltaT);
		sprintf(str,"0");
		fl_set_input( fd_aavso->timeAvg, str );
		fl_set_object_label( fd_aavso->Unscanned, LS(234) );
		fl_set_button( fd_aavso->Unscanned, 1 );
		fl_set_object_label( fd_aavso->WindowList, LS(235) );
		fl_set_button( fd_aavso->WindowList, 1 );
	}
	fl_show_form( fd_aavso->aavso, FL_PLACE_MOUSE, FL_FULLBORDER, LS(230) );
}

void cb_AAVSO (FL_OBJECT *ob, long mode)
{
	char  	filename[256], zeile[256], observer[10];
	char     str[64], lastDateStr[20], useObserver[20];
	FILE		*fDat, *fMissing = NULL;
	struct RHell	*hellPtr;
	struct RDisk	*diskPtr,*diskLoadOld, *lastDiskPtr = NULL;
	TTimeZone	 	old_JulDatZone = JulDatZone;

	int valNamePos, valNameLen, valDatePos, valDateLen, 
	    valMagPos, valMagLen, valMagLower, valObsPos, valObsLen;
	bool valMagTenth  = FALSE;
	bool addtoRecent  = FALSE;
	bool noteLastLine = FALSE;
	bool printUnscannedLines = fl_get_button(fd_aavso->Unscanned);
	bool showWindow = fl_get_button(fd_aavso->WindowList);
		
	float importAvg = atof( fl_get_input(fd_aavso->timeAvg) );
  	strcpy(filename,fl_get_input(fd_aavso->FileName));
	if (filename[0]=='~') 
	{
		strcpy(zeile,filename+1);
		strcpy(filename,pfadHome);
		strcat(filename,"/");
		strcat(filename,zeile);
	}
	fl_hide_form(fd_aavso->aavso);

	fDat = fopen(filename,"r");
  	if (fDat==NULL)  
  	{
		mode = 0;
  	}
 	if (mode==0) 
 	{	/* Cancel */
 		if (fDat)
 			fclose(fDat);
  		fl_activate_all_forms();
   	return;
 	}  
	if (printUnscannedLines) 
	{
		strcat(filename,".missing");
		fMissing = fopen(filename,"w");
	}
	useObserver[0] = 0;
		
	switch ( fl_get_choice( fd_aavso->FileType ) ) {
	case 1:	// AAVSO raw
		valNamePos = 13;
		valNameLen = 10;
		valDatePos = 30;
		valDateLen = 14;
		valMagPos  = 46;
		valMagLen  = 7;
		valObsPos  = 60;
		valObsLen  = 3;
		valMagLower= -1;
		break;
	case 2:	// AAVSO sum
		valNamePos = 8;
		valNameLen = 9;
		valDatePos = 18;
		valDateLen = 12;
		valMagPos  = 31;
		valMagLen  = 7;
		valObsPos  = 62;
		valObsLen  = 3;
		valMagLower= -1;
		break;
	case 3:	// KSOLOREP report by AAVSO, 23.03.01
		valNamePos = 10;
		valNameLen = 9;
		valDatePos = 20;
		valDateLen = 11;
		valMagPos  = 33;
		valMagLen  = 5;
		valObsPos  = -1;
		valObsLen  = 0;
		valMagLower= 32;
		noteLastLine = TRUE;
		{	bool header = TRUE;
			while (header) {
				if (fgetline(zeile,sizeof(zeile),fDat)==NULL)
					header = FALSE;
				else 
				if (strncmp(zeile,"----------",10)==0)
					header = FALSE;
				else 
				{
					char *ptr = strstr(zeile,"initials:");
					if (ptr) 
					{
						ptr += 9;
						while (*ptr && *ptr==' ')	ptr++;
						if (ptr) 
						{
							strncpy(useObserver,ptr,3);
							useObserver[3] = 0;
							CheckObserver( useObserver );
							fprintf(stderr,"KSOLOREP Observer: %s\n",useObserver);
						}
					}
				}
			}
		}
		break;
	case 4:	// first line
		if (fgetline(zeile,sizeof(zeile),fDat)==NULL) 
		{
			fclose(fDat);
	  		fl_activate_all_forms();
			return;
		} 
		else 
		{
			int pos = 0;
			toupperstr(zeile);
			valNamePos = -1;
			valDatePos = -1;
			valMagPos  = -1;
			valMagLower= -1;
			valObsPos  = -1;
			valObsLen  = 0;
			while (zeile[pos]) 
			{
				switch (zeile[pos]) {
				case 'N':
					if (valNamePos<0) 
					{
						valNamePos = pos;
						valNameLen = 1;
					}
					else
						valNameLen++;
					break;
				case 'A':
					addtoRecent = TRUE;
					break;
				case 'D':
					if (valDatePos<0) 
					{
						valDatePos = pos;
						valDateLen = 1;
					}
					else
						valDateLen++;
					break;
				case 'T':
					valMagTenth = TRUE;
					// like 'M'
				case 'M':
					if (valMagPos<0) 
					{
						valMagPos = pos;
						valMagLen = 1;
					}
					else
						valMagLen++;
					break;
				case 'L':
					valMagLower = pos;
					break;
				case 'O':
					if (valObsPos<0) 
					{
						valObsPos = pos;
						valObsLen = 1;
					}
					else
						valObsLen++;
					useObserver[0] = '\0';
					break;
				case '=':
					pos++;
					valObsPos = 0;
					while (zeile[pos]!=' ' && zeile[pos]!='\n' && zeile[pos]!=0)
					{
						useObserver[valObsPos] = zeile[pos];
						pos++;
						valObsPos++;
					}
					useObserver[valObsPos] = '\0';
					CheckObserver( useObserver );
					fprintf(stderr,"Direct Observer: %s\n",useObserver);
					valObsPos = -1;
					valObsLen = 0;
					break;
				}
				pos++;
			}
		}
		if ((valNamePos<0 && !addtoRecent) || valDatePos<0 || valMagPos<0) 
		{
			fclose(fDat);
			fl_show_messages( LS(233) );
	  		fl_activate_all_forms();
			return;
		}
		if (addtoRecent && diskLoad==NULL) 
		{
			/*
			fl_show_messages( "There must be a star to add data!" );
			fclose(fDat);
	  		fl_activate_all_forms();
			return;
			*/
		}
		break;
	default:
		fclose(fDat);
  		fl_activate_all_forms();
		return;
	}

	if (showWindow)
		OpenImportList("AAVSO Data");
		
 	diskLoadOld = diskLoad;
	JulDatZone = _UT;
 	/* Daten durchgehen */

	while (fgetline(zeile,sizeof(zeile),fDat)!=NULL) 
	{
		bool includedData = FALSE;
		bool isDoubleObs  = FALSE;
		toupperstr(zeile);
      diskLoad = NULL;
      diskPtr  = diskFirst->next;
      if (addtoRecent) 
      {
      	diskPtr = diskLoadOld;
      	diskLoad = diskPtr;
      	goto AddData;
      }
      while (diskPtr!=NULL) 
      {
      	int nFound = FALSE;
      	int valNamePos0 = 0;
      	while (zeile[valNamePos+valNamePos0]==' ' && valNamePos0<valNameLen)	
      		valNamePos0++;
      	valNamePos0 += valNamePos;
         sprintf(str,"%s%s ",diskPtr->stb,diskPtr->index);
         nFound = strncmp(zeile+valNamePos0,str,strlen(str));
         if (nFound!=0) 
         {
            sprintf(str,"%s %s ",diskPtr->stb,diskPtr->index);
            nFound = strncmp(zeile+valNamePos0,str,strlen(str));
         }
         if (nFound!=0) 
         {
            sprintf(str,"%s %s ",diskPtr->index,diskPtr->stb);
            nFound = strncmp(zeile+valNamePos0,str,strlen(str));
         }
         if (nFound!=0) 
         {
            sprintf(str,"%s%s",diskPtr->index,diskPtr->stb);
            nFound = strncmp(zeile+valNamePos0,str,strlen(str));
         }
         if (nFound!=0 && noteLastLine) 
         {
            sprintf(str,"%-*s%s ", valNameLen-3, diskPtr->index,diskPtr->stb);
            nFound = strncmp(zeile+valNamePos0,str,strlen(str));
         }
         if (nFound==0) 
         {	/* Name gefunden an Position ptr+1 */
         	AddDataLoad:
		    	diskLoad = diskPtr;
		    	DatenLoeschen();
		    	Daten_Laden_Stern();
		    	
		    	AddData:	// goto this line if addtoRecent
		    	hellPtr = HellNew();
		    	if (noteLastLine) 
		    	{
		    		int i;
		    		for (i=0; i<valDateLen && zeile[valDatePos+i]==' ' && lastDateStr[i]!=0; i++)
		    			zeile[valDatePos+i] = lastDateStr[i];
		    	}
				{	// replace comma by dot
					int i = valDatePos;
					while (zeile[i]!=' ' && zeile[i]!='\0')
					{
						if (zeile[i]==',')	zeile[i] = '.';
						i++;
					}
					
				}
    			hellPtr->datum = atof(zeile+valDatePos);
    			if (hellPtr->datum<2000000)
    				hellPtr->datum += 2400000;
				if (noteLastLine) 
				{
					strncpy( lastDateStr, zeile+valDatePos, valDateLen );
					lastDateStr[valDateLen] = 0;
				}
				//if (zeile[15]==' ') zeile[15] = '0';
				if (zeile[valMagPos+0]==',')	zeile[valMagPos+0] = '.';
				if (zeile[valMagPos+1]==',')	zeile[valMagPos+1] = '.';
				if (zeile[valMagPos+2]==',')	zeile[valMagPos+2] = '.';
				if (zeile[valMagPos+3]==',')	zeile[valMagPos+3] = '.';
    			hellPtr->hell = atof(zeile+valMagPos)*100.0 + 0.5;
    			if (valMagTenth)
    				hellPtr->hell /= 10;
		    	if (zeile[valMagPos-1]=='<' || 
		    	    (valMagLower>=0 && (zeile[valMagLower]=='<' || zeile[valMagLower]=='>') )
		    	   )
		    		hellPtr->hell += 10000;
      		if (valObsLen>0) 
      		{
      			int pos = valObsLen-1;
					strncpy( observer, zeile+valObsPos, valObsLen );
					observer[valObsLen] = 0;
					while (pos>1) 
					{	// TL 24.02.02: finale Leerzeichen gehren nicht zum Code!
						if (observer[pos]==' ')
							observer[pos] = 0;
						pos--;
					}
					if (pos>1)
						CheckObserver( observer );
				} 
				else
					strcpy(observer,useObserver);
		
				if (diskLoad)
		    		sprintf(zeile, "%s%-3.3s %-5.5s  %s  ", 
		    							(showWindow ? "@f" : ""),
		    	   	            diskLoad->stb, diskLoad->index, 
		    	      	         ZeitDatum(hellPtr->datum,filename) );
		    	else // possible if addtoRecent
		    		sprintf(zeile, "@f %s  ", 
		    	      	         ZeitDatum(hellPtr->datum,filename) );
    			if (hellPtr->hell>9999)
		      	sprintf(filename, "<%5.2f",hellPtr->hell*0.01-100);
    			else 
		      	sprintf(filename, " %5.2f",hellPtr->hell*0.01);
    			strcat(zeile,filename);
		    	strcat(zeile,"   ");
    			strcat(zeile,observer);
    			if (showWindow)
    				ImportListAdd( zeile );
    			else
    				fprintf(stderr,"%s\n",zeile);

		    	hellPtr->bemerkung[0] = '\0';
   			hellPtr->art          = 0;
		   	hellPtr = HellPtrAdd(hellPtr,importAvg);
			AddStrings( &hellPtr->observers, " ", observer );
				if (HellPtrDoppelTest(hellPtr,importAvg)) 
				{
					HellPtrDelete(&hellPtr);
					includedData = FALSE;
					isDoubleObs = TRUE;
				}
				else 
				{
	      		if (!addtoRecent)
						Daten_Speichern();
					includedData = TRUE;
				}
				lastDiskPtr = diskPtr;
      		diskPtr = NULL;	// stop searching for star
      	} 
      	else 
      	{
      		diskPtr = diskPtr->next;
	         if (diskPtr==NULL && noteLastLine && lastDiskPtr!=NULL) {
	         	if (zeile[valNamePos]==' ' && zeile[valNamePos+1]==' ' && zeile[valNamePos+2]==' ') {
	      	   	diskPtr = lastDiskPtr;
   	   	   	goto AddDataLoad;
   	   	   }
         	}
      	}
      } // while diskPtr

      if (!includedData)
      	lastDiskPtr = NULL;
      if (printUnscannedLines && !includedData && !isDoubleObs)
      {	FILE* fOut = (fMissing ? fMissing : stderr);
      	fprintf(fOut,zeile);
      	fprintf(fOut,"\n");
      }
	} // while fgetline
  	fclose(fDat);
	if (fMissing)
		fclose(fMissing);
	
	JulDatZone = old_JulDatZone;
#ifndef NO_FORMS
	if (showWindow)
		CURSOR_NORM(fd_ImportList->ImportList->window);
	fl_activate_all_forms();
#endif

  	diskLoad = diskLoadOld;
	if (!addtoRecent) 
	{
	  	DatenLoeschen();
	  	if ((diskLoad!=NULL) && (diskLoad!=diskFirst)) 
	  	{
   	 	Daten_Laden_Stern();
   		Daten_Text();
		}  
	} 
	else
		Daten_Text();
}

#endif

/****************************************************************************/
/* ExportAveragedData                                                       */
/****************************************************************************/

void EAD_Line( FILE *f, double jd, double hell )
{
	double tag;
	int    tag_h, tag_m;
	char   datum[128];
	
	JulDat2Zeit(jd,datum);
	sscanf(datum+12,"%i.%i",&tag_h,&tag_m);
 	tag = (double)tag_h+(double)tag_m/60;
  	datum[10] = '\0';
	fprintf(f,"%.4s%.2s%.2s",datum+6,datum+3,datum);
	sprintf(datum,"%4.3f",tag/24);
	fprintf(f,"%s",datum+1);
	fprintf(f,"  ");
	fprintf(f,"%4.1f\n",hell*0.01);
}

void ExportAveragedData()
{
	static double tag_average = 5.0;

	FILE   *fExport;
	struct RHell *hellPtr;
	double jd_avg = -1, hell_sum;
	int    hell_n = 0;
	char   filename[128];
	const char   *answer;

#ifndef NO_FORMS
	sprintf(filename,"%g",tag_average);
	answer = fl_show_input(LS(130),filename);
	if (answer==NULL)	return;
	tag_average = atof(answer);
#endif

	if (diskLoad==NULL) 
		sprintf(filename,"%sunknown_avg%f.lst",pfadAblage,tag_average);
	else
		sprintf(filename,"%s%s_avg%f.lst",pfadAblage,diskLoad->name,tag_average);
	fExport = fopen(filename,"w");
	hellPtr = hellFirst;
	while (hellPtr!=NULL) {
		if ((besDatumVon<hellPtr->datum) && (hellPtr->datum<besDatumBis) && BitTest(besArt,hellPtr->art)) 
		{ 	double jd = hellPtr->datum;
      	/*
			switch (JulDatZone) {
	      case _MEZ : jd -= 1/24; break;
   	   case _MESZ:
      	case _OEZ : jd -= 2/24; break;
	      case _EST : jd += 5/24; break;
   	   case _CST : jd += 6/24; break;
      	case _MST : jd += 7/24; break;
	      case _PST : jd += 8/24; break;
   	   }
      	*/
      	if ((long)(jd_avg*0.2) != (long)(jd*0.2) )
      	{	if (hell_n>0)
      			EAD_Line( fExport, jd_avg, hell_sum/(double)hell_n );
				jd_avg = 5.0 * (long)(jd*0.2);
				if (hellPtr->hell<10000) {
					hell_sum = hellPtr->hell;
					hell_n = 1;
				} else {
					hell_sum = 0;
					hell_n = 0;
				}
			} else if (hellPtr->hell<10000) {
				hell_sum += hellPtr->hell;
				hell_n++;
			}
		}
		hellPtr = hellPtr->next;
	}
	fclose(fExport);
	strcat(filename," saved!");
#ifndef NO_FORMS
	fl_set_goodies_font(FL_BOLD_STYLE,FL_LARGE_SIZE);
	fl_show_messages(filename);
#endif
}

/****************************************************************************/
/* _ChangeFileFormat                                                        */
/****************************************************************************/
void _ChangeFileFormat()
{
	struct RDisk*	diskPtr;
	struct RHell*	hellPtr;
	int   i, delCount=0;
	
	double dat0 = Zeit2JulDat("15.11.1858");
	double dat1 = Zeit2JulDat("17.11.1858");

	diskPtr = diskFirst->next;
	while (diskPtr!=NULL) 
	{
//		printf("%s ",diskPtr->name);
		diskLoad = diskPtr;
		
		/* altes Format lesen */
		Daten_Laden_Stern();

		if (stern.name[0]==0 || stern.name[0]==' ')
			if (stern.designation[0]==0)
				printf("EMPTY NAME: %s\n",diskPtr->name);

		/* konvertieren */
		for (hellPtr=hellFirst; hellPtr!=NULL; hellPtr=(hellPtr?hellPtr->next:NULL)) 
		{
			if (/*(dat0<=hellPtr->datum) &&*/ (hellPtr->datum<=dat1) &&
			    hellPtr->observers && strstr(hellPtr->observers,"BHE")!=0
			   ) 
			{
				printf("Removing %s\n",stern.name);
				HellPtrDelete(&hellPtr);
				hellPtr = NULL; //hellFirst;
				delCount++;
				stern.hellZahl--;
			}
		}
			
		/* neues Format schreiben */
		Daten_Speichern();

		diskPtr = diskPtr->next;
	}
	
	printf("Gesamt: %i\n",delCount);
}

