
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <ctype.h>
#include <time.h>

#include "vs_sub.h"
#include "vs_ver.h"
#include "vs_file.h"
#include "vs_extrem.h"
#include "libtl/libtl_string.h"

bool	keyboardNumerikPoint = false;

const char SPACE[128] = "                                                                                                   ";

void cb_nothing(void *ob, long data)
{}

char *ZeitDatum (double jd, char *sz)
{
	double zeit;
	char   s[20];

	switch (formDatum) {
	case _TimeDisplay_Normal:
		JulDat2Zeit(jd,sz);
		break;
	case _TimeDisplay_Astro:
		{	int h,m;
			JulDat2Zeit(jd,s);
			sscanf(s+12,"%i.%i",&h,&m);
  			zeit = ((double)h+(double)m/60.0)/24.0+(double)atoi(s);
  			sprintf(sz,"%.4s/%.2s/%06.3f",s+6,s+3,zeit);
		} 
		break;
  	case _TimeDisplay_JD:
	default:
  		jd += JulDatZeitDifferenz[JulDatZone];
  		sprintf(sz,"%7.3f",jd);
	}
	return sz;
}

char *GetDateTimeInputStr (char *dateStr)
{
	char str[20],str1[20];
	
	switch (useDateMode) {
	case USEDATE_GESTERN:
		GetYesterdayStr(dateStr);
		break;
	case USEDATE_ECHTZEIT:
		GetDateStr(str);
		GetTimeStr(str1);
		sprintf(dateStr, "%s, %.2s.%.2s", str, str1, str1+3);
		break;
	case USEDATE_DATUM:
	default:
		sprintf(dateStr,"%s,", useDateStr);
		break;
	}  
	if (formDatum==_TimeDisplay_Astro) 
	{
		sprintf(str, "%.4s/%.2s/%.2s.", dateStr+6, dateStr+3, dateStr);
		strcpy(dateStr,str);
	}
	return dateStr;
}

double StrDate2JulDat (char *str)
{
	double jd;
	char *ptr; 

	if (keyboardNumerikPoint) 
	{
		if (str[10]==',')
	  		str[10] = '~';
  		while ((ptr=strchr(str,','))!=NULL)
   		*ptr = '.'; 
 	}  
  	if ((str[2]=='.') || (str[4]=='/')) 
   	jd = Zeit2JulDat(str);
  	else if (str[5]=='.') 
    	jd = 2400000+atof(str);
  	else 
   	jd = atof(str);
  	return jd;  
}


void GetDateRangeOfMonth( const char* theMonth, const char* theYear, double* dateFrom, double* dateTo )
{
	char datumStr[20];
	sprintf( datumStr, "01.%.2s.%.4s",theMonth,theYear);
	*dateFrom = Zeit2JulDat(datumStr) +0.5;	// mittags
	JulDat2Zeit( *dateFrom+35, datumStr );
	datumStr[0] = '0';
	datumStr[1] = '1';
	datumStr[10] = 0;	// keine Uhrzeit
	*dateTo = Zeit2JulDat(datumStr) +0.5;	// mittags
}


void DatenLoeschen()
{
	RHell  *hellPtr;
	RSonne *sonnePtr;

	stern.name[0] ='\0';
	stern.typ[0]  ='\0';
	stern.designation[0]='\0';
	stern.karte[0]= '\0';
	stern.periode = 0.0;
	stern.max_jd  = JulDat2000;
	stern.hellMax = 0;
	stern.hellMin = 0;
	stern.pmax    = 2;
	stern.pab     = 98;
	stern.pmin    = 2;
	stern.pan     = 98;
	stern.spektralklasse = 0;
	stern.entf    = 0;
	stern.HTML    = FALSE;
	stern.Alert   = FALSE;
	stern.gruppe  = 0;
	stern.importDeltaT = -1;
	stern.hellZahl= 0;
	while (hellFirst != NULL) 
	{
		hellPtr   = hellFirst;
		hellFirst = hellFirst->next;
		HellDelete(&hellPtr);
	}
	ExtremDelete();

	sonneZahl = 0;
	while (sonneFirst != NULL) 
	{
		sonnePtr   = sonneFirst;
		sonneFirst = sonneFirst->next;
		free(sonnePtr);
	}

	stern.spektralklasse = 0;
	stern.entf  = 0;
	sehZahl     = 0;
}
  
void AliasLoeschen()
{
	RAlias *alias;
	while (aliasFirst != NULL) 
	{
		alias = aliasFirst;
		aliasFirst = aliasFirst->next;
		free(alias);
	}
	aliasFirst = NULL;
}

int HellJDindiv (double jd, int i_max, int i_min, double i_per, double i_jd,
                 int i_pmax, int i_pab, int i_pmin, int i_pan)
{
	int hr = i_min;
	double pb;
	if (i_per>0) 
	{
		pb = 200*fracp((jd-i_jd)/i_per)-i_pmax*0.5;
		if ((pb<0) || (pb>i_pab+i_pmin+i_pan))
			hr = i_max;
		else 
		{
			if (pb<i_pab)
				hr = i_max+trunc((i_min-i_max)*(0.5-0.5*cos(pb/i_pab*M_PI)));
			else 
			{
				pb -= i_pab;
				if ((pb<i_pmin) || (i_pan==0)) 
					hr = i_min;
				else 
					hr = i_min-trunc((i_min-i_max)*(0.5-0.5*cos((pb-i_pmin)/i_pan*M_PI)));
			}
		}
	}
	if (i_per<0) 
	{
		if (jd<i_jd-7.0) 
			hr = i_min;
		else 
		if (jd<i_jd-1.0) 
			hr = i_min-trunc((i_min-i_max-2)/6.0*(jd-i_jd+7.0));
		else
		if (jd<i_jd) 
			hr = i_max+2;
		else 
		{
			hr = i_max+trunc(300.0*sqrt((i_jd-jd)/i_per));
			if (hr>i_min) hr = i_min;
		}
	}
	return hr;
}     

int HellJD (double jd)
{
	int hr = stern.hellMin;
	double pb;
	if (stern.periode>0)
	{
		pb = 200.0*fracp((jd-stern.max_jd)/stern.periode)-stern.pmax*0.5;
		if ((pb<0) || (pb>stern.pab+stern.pmin+stern.pan))
		{
			hr = stern.hellMax;
		}
		else 
		{
			if (pb<stern.pab)
			{
				hr = stern.hellMax+trunc((stern.hellMin-stern.hellMax)*(0.5-0.5*cos(pb/stern.pab*M_PI)));
			}
			else 
			{
				pb -= stern.pab;
				if ((pb<stern.pmin) || (stern.pan==0)) 
					hr = stern.hellMin;
				else 
					hr = stern.hellMin-trunc((stern.hellMin-stern.hellMax)*(0.5-0.5*cos((pb-stern.pmin)/stern.pan*M_PI)));
			}
		}
	} 
	else 
	if (stern.periode<0) 
	{
		if (jd<stern.max_jd-7.0) 
			hr = stern.hellMin;
		else 
		if (jd<stern.max_jd-1.0) 
			hr = stern.hellMin-trunc((stern.hellMin-stern.hellMax-2)/6.0*(jd-stern.max_jd+7.0));
		else 
		if (jd<stern.max_jd)
			hr = stern.hellMax+2;
		else 
		{
			hr = stern.hellMax+trunc(300.0*sqrt((stern.max_jd-jd)/stern.periode));
			if (hr>stern.hellMin) hr = stern.hellMin;
		}
	}
	return hr;
}     

RHell* HellNew()
{
	struct RHell* hellPtr = (struct RHell *) malloc(sizeof(struct RHell));
	hellPtr->datum = 0;
	hellPtr->hell  = 9999;
	hellPtr->art	= 0;
	hellPtr->bemerkung[0] = '\0';
	hellPtr->observers    = NULL;
	hellPtr->next	= NULL;
	return hellPtr;
}

void HellDelete( struct RHell** hellPtr )
{
	if ((*hellPtr)->observers)
		free((*hellPtr)->observers);
	free(*hellPtr);
	*hellPtr = NULL;
}

void HellPtrDelete( struct RHell** hellPtr )
{
	if (*hellPtr==hellFirst)
		hellFirst = (*hellPtr)->next;
	else 
	{
		struct RHell *vorPtr = hellFirst;
		while (vorPtr->next && vorPtr->next!=*hellPtr)
			vorPtr = vorPtr->next;
		if (vorPtr->next)
			vorPtr->next = (*hellPtr)->next;
	}
	HellDelete(hellPtr);
}

RHell *HellPtrNr (int nr)
{
	RHell *hellPtr;
	hellPtr = hellFirst;
	while ((nr>1) && (hellPtr!=NULL)) 
	{
		nr--;
		hellPtr = hellPtr->next;
	}
	return hellPtr;
}

RHell* ErzeugeHellMittel (RHell *anPtr, RHell *hellPtr)
{
	if ((anPtr->hell > 9999) && (hellPtr->hell < 10000)) 
	{
		anPtr->hell  = hellPtr->hell;
		anPtr->datum = hellPtr->datum;
		anPtr->art   = 0;
	}
	else 
	if ( ((anPtr->hell > 9999) && (hellPtr->hell > 9999))
	    || ((anPtr->hell < 10000) && (hellPtr->hell < 10000))) 
	{
		anPtr->hell = trunc(((long)(anPtr->hell)*(1+anPtr->art/32)+(long)(hellPtr->hell))
		              /(2+anPtr->art/32));
		anPtr->art += 32;
	}
fprintf(stderr, "Mittel %10.10s JD=%12.3f:  %s + %s\n", stern.name, anPtr->datum, anPtr->observers, hellPtr->observers );
	AddStrings( &anPtr->observers, " ", hellPtr->observers );
	stern.hellZahl--;
	HellDelete(&hellPtr);
	return anPtr;
}

RHell* HellPtrAdd (RHell *hellPtr, double hellMittel)
{
	RHell *lastPtr,*vorPtr;

	flagNeueDaten = TRUE;
	stern.hellZahl++;
	if (hellFirst==NULL) 
	{	// dies ist die erste Beobachtung
		hellFirst = hellPtr;
		hellPtr->next = NULL;
	}
	else 
	{
		bool isBAVobserverLast = false;
		bool isBAVobserverVor  = false;
		lastPtr = hellFirst;
		vorPtr = NULL;
		if (hellRecent) 
		{
			if (hellPtr->datum > hellRecent->datum && hellRecent->next!=NULL) 
			{	// 18.02.01: damit es schneller geht, werden nur die letzten 100 Beobachtungen durchsucht
				if (hellPtr->datum > hellRecent->next->datum) 
				{
					lastPtr = hellRecent->next;
					vorPtr = hellRecent;
				}
			}
		}
		while ((hellPtr->datum > lastPtr->datum) && (lastPtr->next!=NULL)) 
		{
			vorPtr  = lastPtr;
			lastPtr = lastPtr->next;
		}
		// lastPtr ist jetzt entweder das Ende der Liste 
		// oder liegt zwischen den letzten beiden Beobachtungen
		
		isBAVobserverLast = IsObservationInOrg(lastPtr,organisationBAV,NULL,NULL);
		isBAVobserverVor  = IsObservationInOrg( vorPtr,organisationBAV,NULL,NULL);
		if (canAverageBAVobs) 
		{
			isBAVobserverLast = false;
			isBAVobserverVor  = false;
		}
		//	fprintf(stderr,"Kein Mittelwert mit BAV: %s  %10.3f  %s  %4.2f\n",
		//	        stern.name, lastPtr->datum, lastPtr->observers, lastPtr->hell*0.01 );

		if (hellPtr->datum > lastPtr->datum) 
		{	// neue Beobachtung liegt nach der vorhandenen
			if ((hellPtr->datum > lastPtr->datum+hellMittel) || ((lastPtr->art & 31)>0) || (hellMittel < 0.001) || isBAVobserverLast) 
			{	// keine Mittelwertbildung, also die Beobachtung hinzufgen
				lastPtr->next = hellPtr;
				hellPtr->next = NULL;
			}
			else 
			{	// Mittel mit der letzten Beobachtung in der Liste
				hellPtr = ErzeugeHellMittel(lastPtr,hellPtr);
			}
		}
		else
		{	// neue Beobachtung liegt zwischen vorPtr und lastPtr
			if ((hellPtr->datum > lastPtr->datum-hellMittel) && ((lastPtr->art & 31)==0) && (hellMittel >= 0.001) && !isBAVobserverLast)
			{	// Mittelwertbildung, neue Beobachtung liegt kurz vor der vorhandenen
				hellPtr = ErzeugeHellMittel(lastPtr,hellPtr);
			}
			else 
			{
				hellPtr->next = lastPtr;
				if (vorPtr==NULL) 
				{	// neues erstes Listenelement
					hellFirst = hellPtr;
				}
				else 
				{
					if ((hellPtr->datum < vorPtr->datum+hellMittel) && ((vorPtr->art & 31)==0) && (hellMittel >= 0.001) && !isBAVobserverVor) 
					{	// Mittelwertbildung, neue Beobachtung liegt nach der vorhandenen
						hellPtr = ErzeugeHellMittel(vorPtr,hellPtr);
					}
					else
					{
						vorPtr->next = hellPtr;
					}
				}
			}
		}
	}
	return hellPtr;
}

bool HellPtrDoppelTest( struct RHell* hellPtr, float deltaT )
{
	struct RHell *hp = hellPtr;
	/* TL 09.06.01: Test auf Doppeleintrag der Beobachtung */
	if (deltaT==0 && hp->next && hp->next->datum==hp->datum
	    && hp->next->observers && strstr(hp->next->observers,hp->observers))
	{	// diesen Eintrag gibt es schon, also lschen und Warnung ausgeben
		fprintf(stderr,"Verdacht auf Doppeleintrag: %s  %10.3f  %s  %4.2f\n",
		        stern.name, hp->datum, hp->observers, hp->hell*0.01 );
		// Helligkeit ueberschreiben
		//hp->next->hell = hp->hell;
		return TRUE;
	}
	return FALSE;
}

struct RHell* HellPtrTest( struct RHell* hellPtr, const char* observer, float deltaT )
{	// TL 10.10.2004
	double jd0 = hellPtr->datum - deltaT;
	double jd1 = hellPtr->datum + deltaT;
	struct RHell *hp = hellFirst;
	while (hp && hp->datum<jd1)
	{
		if (hp->datum>jd0)
		{
			if (hp->observers && strstr(hp->observers,observer))
				return hp;
		}
		hp = hp->next;
	}
	return NULL;
}

int BitTest (int src, int test)
{
	int ok = FALSE;
	test &= 31;
	if (((src & 1)>0) && (test==0)) ok = TRUE;
	if (((src & 2)>0) && (test==1)) ok = TRUE;
	if (((src & 4)>0) && (test==2)) ok = TRUE;
	return ok;
} 

void MailFile (char *mailTO, char *nameFile)
{
	char zeileStr[256];
	if (mailTO[0]!='\0') 
	{
		sprintf(zeileStr, "%s < %s\n", mailTO, nameFile);
		system(zeileStr);
		fprintf(stderr,VerError);
		fprintf(stderr,zeileStr);
	}	
}

