#ifndef __VS_SUB_H
#define __VS_SUB_H

#include "libtl/astrotools.h"

typedef enum 
	{
		STG_NONE,
		STG_MIRA,
		STG_IRR,
		STG_ERUP,
		STG_NOVA,
		STG_BED,
		STG_PULS,
		STG__LAST
	} TypStarGroup;
				
#define StrLenName 		50
#define StrLenTyp			256
#define StrLenBemerkung 15
#define StrLenKarte 		14
#define StrLenObsCode	16
#define StrLenPfad	512

typedef struct RObserver 
{
   char	code[StrLenObsCode];
   char	name[StrLenName];
   char* country;
   char* instruments;
   char*	email;
   bool	inList;	/* for use in ObserverEdit */
   struct RObserver*	next;
} RObserver;

typedef struct ROrganisation 
{
	char	code[StrLenObsCode];
	char*	name;
	struct ROrganisation* next;
} ROrganisation;

typedef struct RStern 
{
	char     name[StrLenName],
	         typ[StrLenTyp],
	         designation[StrLenName],
	         karte[StrLenKarte];
   int     	hellMax,hellMin, 
   			pmax,pab,pmin,pan, 
   			spektralklasse;
   double  	periode, max_jd, entf, importDeltaT;
   TypStarGroup	gruppe;
   bool		HTML;
   bool		Alert;
   int     	hellZahl;
   int		extremZahl;
} RStern;
     
typedef struct RHell 
{
  double 	datum;
  int 	 	hell;
  int 	 	art; /* 0:unbek.,1:vis.,2:fotog.,DIV32=0:unbek.&Mittelwert aus xMOD32+1 */
  char 	 	bemerkung[StrLenBemerkung];
  char*		observers;
  struct RHell* next;
} RHell;

typedef struct RSonne 
{
  double 	datum;
  int			gruppe, fleck, seh, rel, relZeig;
  struct RSonne *next;
} RSonne;

typedef struct RDisk 
{
  char          name[32];
  char          stb[4]; 
  char          index[32];
  struct RDisk* next;
} RDisk;

typedef struct RAlias 
{
  char	         muster[32], org[32];
  struct RAlias*	next;
} RAlias;

typedef struct RWinpos 
{
	int x,y,w,h;
} RWinpos;

struct RStern	stern;
struct RHell	*hellFirst, *hellRecent;
struct RDisk	*diskFirst;
struct RDisk	*diskLoad;
struct RSonne	*sonneFirst;
int     			sonneZahl,sehZahl;
char    			seh_name[10][80];
double  			seh_k[10];
int     			seh_z[10];
bool 				flagNeueDaten;

char	bavObserver[32],
		bavStreet[32],
		bavCity[32],
		bavCountry[32],
		bavInstrument[50],
		bavZip[12],
		initialAAVSO[6],
		initialBAV[6],
		initialAFOEV[6],
		standardObserver[20];

double				besDatumVon,besDatumBis;
int     				besArt;
bool	 				mitNegativBeob,mitTheoretKurve;
bool					tageslisteMitNegativ,tageslisteMitIdent;
TTimeDisplay		formDatum;
int     				monatEinschubX, monatEinschubY;
bool 					monatEinschubNeu;
double				importDeltaT;
bool					canAverageBAVobs;
int     				grafikDruckFormat, grafikDruckX, grafikDruckY,
						grafikDruckEinschubX, grafikDruckEinschubY;
bool 					autoScanVSNET;
struct RAlias*		aliasFirst;
struct RObserver* observerFirst;
struct ROrganisation*	organisationFirst;
struct ROrganisation*	organisationBAV;
bool					observerChange;

extern bool			keyboardNumerikPoint;
bool					saveWinpos;
struct RWinpos 	fd_bav_kurve_wp, fd_beob_list_wp, fd_chart_wp,
						fd_graf_hell_wp, fd_graf_phase_wp, fd_perana_wp;

#define USEDATE_GESTERN  0
#define USEDATE_ECHTZEIT 1
#define USEDATE_DATUM    2
char    useDateStr[20];
int     useDateMode;


typedef enum {
	SCRIPT_NO = 0,
	SCRIPT_BEOB_NEU,
	SCRIPT_CHART,
	SCRIPT_GRAFIK,
	SCRIPT_MAX_LISTE,
	SCRIPT_MONAT,
	SCRIPT_TAG,
	SCRIPT_VSNET,
	SCRIPT_BAV_DB,
	SCRIPT_HTML_STERN,
} ScriptMode_t;
bool		scriptRun;
ScriptMode_t	scriptMode;
char		scriptArg[512];

char		pfadHome[StrLenPfad], pfadStart[StrLenPfad], pfadFont[StrLenPfad], pfadSTV[StrLenPfad],
		pfadChart[256], pfadAblage[256], pfadTMP[256], pfadGCVS[256],
		pfadVSNET[256], pfadObserver[256], pfadOrganisation[256],
		pfadBAV[StrLenPfad],
		linkIMG[StrLenPfad];
char    	mailAAVSO[80], mailAFOEV[80], mailBAV[80], mailVSNET[80];

void cb_nothing();

char*		GetDateTimeInputStr (char *dateStr);
double 	StrDate2JulDat (char *str);
char*		ZeitDatum   (double jd, char *sz);
void		GetDateRangeOfMonth( const char* theMonth, const char* theYear, double* dateFrom, double* dateTo );

void   	DatenLoeschen();
void   	AliasLoeschen();
int    	HellJDindiv (double jd, int i_max, int i_min, double i_per, double i_jd,
                      int i_pmax, int i_pab, int i_pmin, int i_pan);
int    	HellJD(double jd);
RHell*	HellNew();
void 		HellDelete( struct RHell** hellPtr );		// RHell lschen
void 		HellPtrDelete( struct RHell** hellPtr );	// RHell aus hellFirst-Liste entfernen
RHell*	HellPtrNr(int nr);
RHell*  	HellPtrAdd (struct RHell *hellPtr, double hellMittel);
 			/* gibt Pointer auf den neuen Helligkeitseintrags zurueck */
bool	 	HellPtrDoppelTest( struct RHell* hellPtr, float deltaT );
			/* testet bei deltaT=0, ob es bei dem Datum und Beobachter schon einen Eintrag gibt, und schickt dann TRUE zurck */
struct RHell* 	HellPtrTest( struct RHell* hellPtr, const char* observer, float deltaT );
			/* testet im Zeitraum deltaT um hellPtr, ob es von dem Beobachter einen Eintrag gibt, und gibt dann TRUE zurck */

int 		BitTest (int src, int test);

void   	MailFile (char *mailTO, char *nameFile);

/* in varstars.c: */
extern void OpenHelp( const char* titel, const char** text );

#define testmin(s,d) { if (s<d) d=s; }
#define testmax(s,d) { if (s>d) d=s; }

#endif /* __VS_SUB_H */
